Unit Fft;

interface

uses Maths, sysUtils;

const
   MaxVecteurFFT = 8192;

type
   tCalculFourier = (cZero,cHamming,cHarmonique);
   IndiceVecteurFFT = 0..MaxVecteurFFT;
   VecteurFFT = array[IndiceVecteurFFT] of double;
   VecteurIntFFT = array[indiceVecteurFFT] of integer;

var
   AnalyseHarmonique : boolean;

Procedure Fourier(Nbre : integer;const X : vecteur;var Xr,Xi : vecteurFFT;
                  calculFourier : TcalculFourier;var NbreFFT : integer);
Procedure SpectreAmplitude(Nbre : integer;const X : vecteur;
                           var ampli,freq : vecteurFFT;
                           var NbreFFT : integer);
procedure calculAutoCorrelation(Nbre : integer;const X : vecteur;
                                var indexPeriode : integer);

Implementation

var
   CoeffInterpole : double;

function Puiss2Inf(N : LongInt) : LongInt;
begin
    result := 1;
    while result<N do result := 2*result;
    if result>N then result := result div 2;
end;

function Puiss2Sup(N : LongInt) : LongInt;
begin
    result := Puiss2Inf(N);
    if result<N then result := 2*result;
    if result>maxVecteurFFT then result := maxVecteurFFT;
end;

var tbi : vecteurIntFFT;
    wr,wi : vecteurFFT;

Procedure CalculIntLineaire(const Y : vecteur;Nbre,NbreLisse : integer;
          var YLisse : vecteur);
var i,ix : integer;
    t : double;
begin
      Ylisse[0] := Y[0];
      coeffInterpole := Nbre/NbreLisse;
      Ylisse[pred(NbreLisse)] := Y[pred(Nbre)];
      for i := 1 to NbreLisse-2 do begin
          t := i*coeffInterpole;
// recherche de i initial proche
          ix := trunc(t);
// x[ix]<t<=x[ix+1]
          Ylisse[i] := Y[ix]+(t-ix)*(Y[succ(ix)]-Y[ix]);
      end;
end; // CalculIntLineaire

Procedure spectreAmplitude(Nbre : integer;const X : vecteur;
                           var ampli,freq : vecteurFFT;var NbreFFT : integer);
var i : integer;
    Xr,Xi : vecteurFFT;
begin
    if Nbre>maxVecteurFFT then Nbre := maxVecteurFFT;
    if analyseHarmonique
       then Fourier(Nbre,X,Xr,Xi,cHarmonique,NbreFFT)
       else Fourier(Nbre,X,Xr,Xi,cHamming,NbreFFT);
    for i:= 0 to (NbreFFT div 2) do begin
         ampli[i] := sqrt(sqr(xr[i])+sqr(xi[i]));
         freq[i] := i;
    end;
end;

PROCEDURE tfd(var xr,xi : vecteurFFT;NbreFFT : integer);
 { Transformation proprement dite }
var a,i,j,k,k0,nbbloc,tbloc,r,r0,p0,nombre0 : integer;
    zr,zi,c : double;
begin
  nbbloc:=NbreFFT;
  tbloc:=1;
  r:=trunc(0.5+Ln(NbreFFT)/ln(2));
  for i:=1 to r do begin
          tbloc:=2*tbloc;
          nbbloc:=nbbloc div 2;
          for j:=1 to nbbloc do begin
             nombre0:=(NbreFFT div nbbloc -1) div 2;
             r0:=0;
             for k0:=0 to nombre0 do begin
                    k:=k0+(pred(j))*tbloc;
                    p0:=k+tbloc div 2;
                    a:=r0;
                    zr:=xr[p0]*wr[a]-xi[p0]*wi[a];
                    zi:=xr[p0]*wi[a]+xi[p0]*wr[a];
                    xr[p0]:=xr[k]-zr;
                    xi[p0]:=xi[k]-zi;
                    xr[k]:=xr[k]+zr;
                    xi[k]:=xi[k]+zi;
                    r0:=r0+nbbloc;
             end; {k0}
          end; {j}
  end; {i}
  c := 2/NbreFFT;
  for i := 0 to pred(NbreFFT) do begin
       xr[i]:=c*xr[i];
       xi[i]:=c*xi[i];
  end;
  xr[0] := xr[0]*2;
  xi[0] := xi[0]*2;
end; // tfd

PROCEDURE tfdi(var xr,xi : vecteurFFT;NbreFFT : integer);
 { Transformation inverse proprement dite }
var a,i,j,k,k0,nbbloc,tbloc,r,r0,p0,nombre0 : integer;
    zr,zi,c : double;
begin
  nbbloc:=NbreFFT;
  tbloc:=1;
  r:=trunc(0.5+Ln(NbreFFT)/ln(2));
  for i:=1 to r do begin
          tbloc:=2*tbloc;
          nbbloc:=nbbloc div 2;
          for j:=1 to nbbloc do begin
             nombre0:=(NbreFFT div nbbloc -1) div 2;
             r0:=0;
             for k0:=0 to nombre0 do begin
                    k:=k0+(pred(j))*tbloc;
                    p0:=k+tbloc div 2;
                    a:=r0;
                    zr:=xr[p0]*wr[a]+xi[p0]*wi[a];
                    zi:=-xr[p0]*wi[a]+xi[p0]*wr[a];
                    xr[p0]:=xr[k]-zr;
                    xi[p0]:=xi[k]-zi;
                    xr[k]:=xr[k]+zr;
                    xi[k]:=xi[k]+zi;
                    r0:=r0+nbbloc;
             end; {k0}
          end; {j}
  end; {i}
  c := 2/NbreFFT;
  for i := 0 to pred(NbreFFT) do begin
       xr[i]:=c*xr[i];
       xi[i]:=c*xi[i];
  end;
  xr[0] := xr[0]*2;
  xi[0] := xi[0]*2;
end; // tfdi

PROCEDURE inverserIndices(NbreFFT : integer);
{ Table de l'ordre inverse des bits }
var i,j,k,r,p0,q,tiroir : integer;
    dpsn : double;
begin
  dpsn:=2*pi/NbreFFT;
  for i:=0 to pred(NbreFFT) do begin
         wr[i]:=COS(dpsn*i);
         wi[i]:=SIN(-dpsn*i);
// ATTENTION : les indices ne sont pas permuts
// la permutation selon tbi[] sera faite dans le fentrage 
  end;
  for i:=0 to pred(NbreFFT) do tbi[i]:=i;
  r:=trunc(0.5+Ln(NbreFFT)/ln(2));
  for i:=0 to pred(NbreFFT) do begin
      p0:=r;
      q:=1;
      k:=0;
      for j:=1 to r do begin
           if (i and puissDeux(PRED(p0)))<>0 then k:=k+q;
           q:=2*q;
           DEC(p0);
      end;
      if k>i then begin
            tiroir:=tbi[k];
            tbi[k]:=tbi[i];
            tbi[i]:=tiroir;
      end;
    end;
end; // inverserIndices

Procedure Fourier(Nbre : integer;const X : vecteur;var Xr,Xi : vecteurFFT;
                  calculFourier : TcalculFourier;var NbreFFT : integer);

procedure fenetrerHamming;
const coeff = 1.852;
//    FlatTop : coeffFenetrage := 4.638;
var j : integer;
    kf,dpsn : double;
begin
     dpsn:=2*pi/Nbre;
     for j:=0 to pred(Nbre) do begin
         kf:=0.54-0.46*cos(j*dpsn);
         xr[tbi[j]]:=kf*X[j]*coeff; // fentrage et renormalisation
// et rangement avec permutation des mesures tableau fentres dans xr[]
         xi[j]:=0;
     end;
     for j:=Nbre to pred(NbreFFT) do begin
         xr[tbi[j]]:=0;
// rangement avec permutation des zros dans xr[] 
         xi[j]:=0;
     end;
end; // Hamming

procedure fenetrerRect;
var i : integer;
begin
        for i:=0 to pred(Nbre) do begin
            xr[tbi[i]] := x[i];
            xi[i] := 0;
        end;
        for i:=Nbre to pred(NbreFFT) do begin
            xr[tbi[i]] := 0;
            xi[i] := 0;
        end;
end;

var Xlisse : vecteur;
    i : integer;
    coeff : double;
begin
     if Nbre>maxVecteurFFT then Nbre := maxVecteurFFT;
     case calculFourier of
        cZero : begin
            NbreFFT := puiss2Sup(nbre);
            inverserIndices(NbreFFT);
            fenetrerRect;
        end;
        cHarmonique : begin
            NbreFFT := puiss2Sup(Nbre);
            CalculIntLineaire(X,nbre,NbreFFT,XLisse);
            inverserIndices(NbreFFT);
            for i := 0 to pred(NbreFFT) do begin
                  xr[tbi[i]] := Xlisse[i];
                  xi[i] := 0;
            end;
        end;
        cHamming : begin
            NbreFFT := puiss2Sup(Nbre);
            inverserIndices(NbreFFT);
            fenetrerHamming;
        end;
     end; // case calculFourier
     tfd(xr,xi,NbreFFT);
     if calculFourier=cHamming then begin
         coeff := NbreFFT/Nbre;
         for i := 0 to pred(NbreFFT) do begin
            xr[i] := xr[i]*coeff;
            xi[i] := xi[i]*coeff;
         end;
     end;
end; // Fourier

procedure calculAutoCorrelation(Nbre : integer;const X : vecteur;
                                var indexPeriode : integer);
var Xr,Xi,Xa,Ya : vecteurFFT;
    i,j,nbreFFT : integer;
    moy : double;
begin
    if Nbre>maxVecteurFFT then Nbre := maxVecteurFFT;
    Fourier(Nbre,X,Xr,Xi,cZero,nbreFFT);
    for i := 0 to pred(NbreFFT) do begin { conjugue(x)*x }
        Xr[i] := Xr[i]*Xr[i]+Xi[i]*Xi[i];
        Xi[i] := 0;
    end;
//  effectuer fenetre inverse
    for j := 0 to pred(NbreFFT) do begin
         Xa[tbi[j]] := xr[j];
         Ya[tbi[j]] := xi[j];
     end;
    tfdi(Xa,Ya,nbreFFT);
    moy := 0;
    for j := 0 to pred(NbreFFT) do
        moy := moy + Xa[j];
    moy := moy/nbre;
    indexPeriode := 0;
    i := 0;
    repeat
        inc(i)
    until (i>Nbre) or (Xa[i]<moy); // on vite le continu
    for j := i+1 to (NbreFFT div 2) do
        if Xa[j]>moy then begin
           moy := Xa[j];
           indexPeriode := j;
        end;
    if indexPeriode<32 then indexPeriode := 0;
end; // calculAutoCorrelation

end.

