unit Graphe;

interface

uses windows, SysUtils, Graphics, Classes,
   Forms, Controls, Buttons, StdCtrls, ExtCtrls, Dialogs, inifiles,
   maths, gtsUtil, uniteKer, gtsdef, constgts, Math, parser10,
   Printers, jpeg, clipBrd, fft, aideGTS2;

const
   IntSansSignif         = -1;
   dimPoint: integer     = 2;
   scaleCurseur          = 10;
   LongNombreAxe         = 4;
   NbreGraduations       = 10;
   NbreGradMin           = 5;
   NbreRampeMax          = 6;
   Fleche: array[boolean] of char = (WideChar($2193),WideChar($2191));
   GammedB = 50; // facteur 0,3 %
   GammeFourier = 0.003;
   CouleurRepere: Tcolor = clBlue;

type
   Ttrace = (trLigne,trPoint,trRaie,trHisto,trSpiro,trCardiaque);
   sTrace = set of Ttrace;

   Trepere = class
      Valeur: double;
      edit: Tedit;
      destructor Destroy;override;
   end;

   TvaleurMagnum = record
      case boolean of
         False: (octets: array[0..5] of AnsiChar);
         True: (DateM: longword;
            posM: word);
   end;

   TmodeFront   = (mFrequence, mComptage);
   Tcurseur     = (curNone, curSynchro, curPreAcq);

   TCourbe = class(Tvoie)
   private
      a, b: double; { entier = rel*a+b }
   public
      IndexDernier: integer;
      deltaAxe, exposant, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      axeLog, axeDateTime: boolean;
      NbreChiffre: integer;
      courbeBottom, courbeTop: integer;
      LabelVoie: Tlabel;
      GBvoie: TgroupBox;
      AbscisseBtn : TbitBtn;
      miniLoupe,maxiLoupe : double;
      constructor Create(Anom: string; Aunite: string; Asignif: string;
         Anumero: integer);
      destructor Destroy; override;
      procedure setDelta(grandAxe: boolean; min, max: double);
      procedure chercheMinmax(Nbre: integer; var min, max: double);
      function FormatAxe(x: double): string;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
   end;

   TCourbeFourier = class(Tunite)
   private
      a, b: double; { entier = rel*a+b }
   public
      maxi : double;
      couleur : Tcolor;
      active : boolean;
      deltaAxe, exposant, pointDebut: double;
      { deltaAxe = cart entre graduations sur les axes }
      NbreChiffre: integer;
      courbeBottom, courbeTop: integer;
      vecteurAff : vecteurFFT;
      decibel: boolean;
      constructor Create(Anom: string; Aunite: string; Asignif: string);
      procedure setDelta(grandAxe: boolean);
      function FormatAxe(x: double): string;
      function XWindow(r: double): integer;
      function XMonde(xi: integer): double;
   end;

   TvoieNonEA = class(Tcourbe)
      procedure setMinMax; override;
   end;

   TsortieAnalogique = class(TvoieNonEA)
   protected
      function GetNomVoieOrphy: string; override;
   public
      DebutSortie, FinSortie: array[1..NbreRampeMax] of smallint;
      DeltatSA:    integer; { ms }
      SAi:         ansichar;
      NbreInterSA: integer;
      UseGBF:      boolean;
      Base32Micro: boolean;
      vecteurUtilisateur: vecteur;
      NbreRampes:  integer;
      nbrePulse, largeurPulse, distancePulse, debutPulse: integer;
      constructor Create;
      procedure calcVecteur;
      procedure calcVecteurNerf(NbrePoints: integer);
      function ValeurVolt(V: integer): double; override;
      function calibreVoie: string; override;
      function ValeurReal(V: integer): double; override;
      function CodeWGO: string;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
   end;

   TentreeClavier = class(TvoieNonEA)
      function GetNomVoieOrphy: string; override;
      constructor Create;
   end;

   TvoieCalculee = class(TvoieNonEA)
   protected
      VoltEAParser: array[indiceVoie] of PParserFloat;
      procedure setExpression(Avalue: string); override;
      function GetNomVoieOrphy: string; override;
   public
      function CalcValeur: double;
      function calibreVoie: string; override;
      procedure recopieC(N: integer);
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      constructor Create;
   end;

   TsortiePWM = class(TvoieNonEA)
   protected
      Ffrequence: integer; { en Hz }
      Fduty100:   integer; { en % }
      Fperiode:   integer; { 0..65535 }
      Fhorloge:   integer; { 0=16 MHz ... 4=2 MHz }
      FcodeDuty:  integer; { 0..Fperiode }
      FperiodeOK: boolean;
      function GetNomVoieOrphy: string; override;
      procedure setFrequence(Avalue: integer);
      procedure setDuty(Avalue: integer);
      procedure setPeriode;
   public
      PWMi: ansichar;
      property Frequence: integer Read Ffrequence Write setFrequence;
      property Duty100: integer Read Fduty100 Write setDuty;
      constructor Create;
      function CodePWM: string;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
   end;

   TbaseDeTemps = class(TvoieNonEA)
   private
      FDureeBalayage: double;
      FPreAcq100:     integer;
      procedure setPreAcq100(Avalue: integer);
      procedure setDureeBalayage(Avalue: double);
      function getNbrePreAcq: integer;
   public
      Deltat:       double;
      ModeLent:     boolean;
      PreAcqPermis: boolean;
      property DureeBalayage: double Read FdureeBalayage Write setDureeBalayage;
      property PreAcq100: integer Read FPreAcq100 Write setPreAcq100;
      property NbrePreAcq: integer Read getNbrePreAcq;
      constructor Create;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
   end;

   Tfrequence = class(TvoieNonEA)
   protected
      function GetNomVoieOrphy: string; override;
   public
      seuil:        integer;
      FreqEA11:     boolean;
      numVoie:      integer;
      miniF, maxiF: double;
      mode:         TmodeFront;
      constructor Create;
      function calibreVoie: string; override;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      function AffVoltmetre: string; override;
      function Commande: String;
      function Stop: String;
   end;

   Tsondor = class(TvoieNonEA)
   protected
      function GetNomVoieOrphy: string; override;
   public
      EFi, SBi, EAi: integer;
      AttenteDeltat: boolean;
      constructor Create;
      function calibreVoie: string; override;
      function Commande(mode: TmodeAcq): string;
   end;

   Tmagnum = class(TvoieNonEA)
   private
      FmodeAngulaire, FmodeRadian, Fchute: boolean;
      procedure setModeAngulaire(Value: boolean);
      procedure setModeRadian(Value: boolean);
      procedure setChute(Value: boolean);
   public
      increment:        double;
      TraitMagnumE:     double; // E=effective
      OscilMin:         integer;
      ArretAuto:        boolean;
      DureeOscillation: double; // s
      isCodeurInc, codeurActif, Negatif: boolean;
      EFi, EBi:         byte;
      PasEntier:        boolean;
      Montant: boolean;
      Synchro: boolean;
      property modeAngulaire: boolean Read FmodeAngulaire Write setModeAngulaire;
      property modeRadian: boolean Read FmodeRadian Write setModeRadian;
      property chute: boolean Read Fchute Write setChute;
      constructor Create;
      function AffVoltmetre: string; override;
      function calibreVoie: string; override;
      function ValeurReal(V: integer): double; override;
      procedure load(Avoie: string; Fichier: tMemIniFile); override;
      procedure store(Avoie: string; Fichier: tMemIniFile); override;
      procedure recopieM(N: integer);
      function codeMagnum: String;
      function stopMagnum: String;
      function codeCodeurInc: String;
      procedure setOrigine(var N: integer);
   end;

  TlisteRepere = class(TList)
  protected
    function Get(Index: integer): TRepere;
    procedure Put(Index: integer; Item: TRepere);
  public
    function Add(Item: TRepere): integer;
    function Remove(Item: TRepere): integer;
    property Items[Index: integer]: TRepere Read Get Write Put; default;
    procedure Clear; override;
  end;

   TgrapheGTS = class
   private
      FNbreInter:   integer;
      FvoieSynchro: indiceVoie;
      procedure setNbreInter(Avalue: integer);
      function getVoieSynchro: indiceVoie;
      procedure setVoieSynchro(avoie: indiceVoie);
      function getNumSynchro: integer;
   public
      Canvas:        Tcanvas;
      ListeRepere:  TListeRepere;
      Trace:  sTrace;
      NbreCourbe:    integer;
      avecAxeX, avecAxeY: boolean;
      grandAxeX, grandAxeY: boolean;
      voieOrphy:     array[indiceVoie] of Tcourbe;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      NbreVoie:      integer;
      NbreAff, NbreCourant: integer;
      voieAbscisse:  indiceVoie;
      seuilSynchro:  integer;
      monocoup:      boolean;
      loupeAuto:     boolean;
      ModeSynchro:   TModeSynchro;
      ModeAcq:       TModeAcq;
      montant:       boolean;
      FrontSynchro:  ansichar;
      recalageZero:  boolean;
      CoeffInterp:   double; { interpolation lie  WPE }
      ReticuleTrace: boolean;
      deltaBouge:    integer;
      voieBouge:     indiceVoie;
      voieFiltreEntree, voieFiltreSortie, voieFiltrePhase: indiceVoie;
      deltatFiltre:  integer;
      DeltaSpiro : double;
      vecteurTemps : vecteur;
      property NbreInter: integer Read FnbreInter Write setNbreInter;
      property voieSynchro: indiceVoie Read getVoieSynchro Write setVoieSynchro;
      property numSynchro: integer Read getNumSynchro;
      constructor Create;
      destructor Destroy; override;
      procedure VerifSynchro;
      function VerifSondor: boolean;
      procedure Draw;
      procedure Print;
      procedure TraceCourbes;
      procedure TraceUnPoint(voie: integer);
      procedure TraceReticule(voie: integer);
      procedure SetHauteurFonte(H: integer);
      procedure tri;
      procedure CourantVersAff;
      function seuilSynchroReal: double;
      procedure setSeuilSynchro(sync: double);
      procedure setOrigineTemps;
      procedure TraceMotif(xi, yi: integer; motif: Tmotif);
      function ordreLanceFiltre: string;
      procedure TraceRepere(repere: Trepere);
      procedure VersBitMap(NomFichier: string; isJpeg: boolean);
      Procedure SetIndexPeriode(voie : indiceVoie;var indexPeriode : integer);
      procedure PenDefault;
   end;

   TgrapheFourier = class
   private
   public
      Canvas: Tcanvas;
      Trace:  sTrace;
      NbreCourbe:    integer;
      avecAxeX, avecAxeY: boolean;
      grandAxeX, grandAxeY: boolean;
      voieT: TcourbeFourier;
      voieOrphy:  array[voieA..voieAnaMax] of TcourbeFourier;
      LimiteCourbe, LimiteFenetre: Trect;
      LargCarac, HautCarac, Marge: integer;
      NbreAff: integer;
      WidthHisto : integer;
      NbreInter:   integer;
      constructor Create;
      destructor Destroy; override;
      procedure Draw;
      procedure Print;
      procedure TraceCourbes;
      procedure SetHauteurFonte(H: integer);
      procedure VersBitMap(NomFichier: string; isJpeg: boolean);
      Function isAxeX(x,y : integer) : boolean;
      procedure changeEchelleX(xnew,xold : integer);
      Procedure PointProche(var x,y : integer;var IndexVoie,IndexPoint : integer);
      procedure PenDefault;
   end;

var
   grapheGts:    TgrapheGTS;
   grapheFFT:    TgrapheFourier;
   sortieAnalogique: TsortieAnalogique;
   sortiePWM:        TsortiePWM;
   frequence:        Tfrequence;
   tension:          Tunite;
   magnum:           Tmagnum;
   sondor:           Tsondor;
   BaseDeTemps:      TbaseDeTemps;
   EntreeClavier:    TentreeClavier;
   voieCalculee:     TvoieCalculee;
   Gene32Permis:     boolean;
   FreqMega, NbreTrame, NbreParVoieParTrame, DecaleMega: integer;

   quadrillage, AnalyseurLogique: boolean;
   avecUnAxe,decibelGlb : boolean;

implementation

const
   HorlogeMaxPWM = 16000000; { 16 MHz }

var
   MaxGr, MinGr: integer;
   Monochrome:    boolean;
   ImpressionEnCours: boolean;

function ColorGr(couleur: Tcolor): Tcolor;
begin
   if monochrome then
      Result := clBlack
   else
      Result := couleur;
end;

function Tcourbe.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function TcourbeFourier.XMonde(xi: integer): double;
begin
   Result := (xi - B) / A;
end;

function Tcourbe.XWindow(r: double): integer;
begin
   if r = sansSignif then
      Result := IntSansSignif
   else if AxeLog then
      if r > 0 then begin
         try
            Result := round(A * Log10(r) + B)
         except
            Result := IntSansSignif
         end;
      end
      else
         Result := IntSansSignif
   else
      Result := round(A * r + B);
end;

function TcourbeFourier.XWindow(r: double): integer;
begin
   if r = sansSignif then
      Result := IntSansSignif
   else Result := round(A * r + B);
end;

procedure TgrapheGTS.TraceMotif(xi, yi: integer; motif: Tmotif);

   procedure Segment(x1, y1, x2, y2: integer);
   begin
      Canvas.moveTo(x1, y1);
      Canvas.lineTo(x2, y2);
   end;

var
   PolyLigne: array[1..5] of Tpoint;
begin
   case motif of
      mCroix: begin
         Segment(xi - dimPoint, yi, xi + dimPoint, yi);
         Segment(xi, yi - dimPoint, xi, yi + dimPoint);
      end;
      mCroixDiag: begin
         segment(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
         segment(xi + dimPoint, yi - dimPoint, xi - dimPoint, yi + dimPoint);
      end;
      mCercle: canvas.ellipse(xi - dimPoint, yi - dimPoint, xi +
            dimPoint + 1, yi + dimPoint + 1);
      mCarre: canvas.rectangle(xi - dimPoint, yi - dimPoint, xi + dimPoint, yi + dimPoint);
      mLosange: begin
         PolyLigne[1] := Point(xi - dimPoint, yi);
         PolyLigne[2] := Point(xi, yi - dimPoint);
         PolyLigne[3] := Point(xi + dimPoint, yi);
         PolyLigne[4] := Point(xi, yi + dimPoint);
         PolyLigne[5] := Point(xi - dimPoint, yi);
         canvas.polyLine(PolyLigne);
      end;
   end;
end;

procedure TgrapheGTS.TraceUnPoint(voie: integer);
var
   xi, yi: integer;
begin
   with voieOrphy[voie] do begin
      canvas.pen.color := colorGr(couleur);
      canvas.brush.color := colorGr(couleur);
      if (modeAcq in [AcqClavier, AcqPoint]) and (NbreAff < 32) then
         canvas.pen.Width := 3;
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
         traceMotif(xi, yi, motif);
      penDefault;
   end;
end; // TracePoint 

procedure TgrapheGTS.TraceReticule(voie: integer);
var
   xi, yi: integer;
begin with voieOrphy[voie] do begin
      xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].valeurAff);
      yi := XWindow(valeurAff);
      if (xi <> IntSansSignif) and (yi <> IntSansSignif) then begin
         canvas.pen.color := colorGr(couleur);
         canvas.moveTo(xi - scaleCurseur, yi);
         canvas.lineTo(xi + scaleCurseur, yi);
         canvas.moveTo(xi, yi - scaleCurseur);
         canvas.lineTo(xi, yi + scaleCurseur);
      end;
   end;
end;{ TraceReticule }

procedure TgrapheGts.SetHauteurFonte(H: integer);
var S : integer;
begin
   S := H * 72 div canvas.Font.PixelsPerInch;
   S := 2 * (S div 2);
   if S <= 8 then S := 8;
   if not (ImpressionEnCours) and (S>18) then S := 18;
   if ImpressionEnCours and (S>24) then S := 24;
   canvas.font.size := S;
   HautCarac := canvas.TextHeight('A');
   LargCarac := canvas.TextWidth('A');
   Marge := largCarac div 2;
end;

procedure TgrapheFourier.SetHauteurFonte(H: integer);
begin
   H := H * 72 div canvas.font.PixelsPerInch;
   H := 2 * (H div 2);
   if H<=8 then H := 8;
   if ImpressionEnCours then begin
        if H>18 then H := 18;
     end
     else begin
        if H>16 then H := 16;
     end;
   canvas.font.Size := H;
   HautCarac := canvas.TextHeight('A');
   LargCarac := canvas.TextWidth('A');
   Marge := largCarac div 2;
end;

procedure TgrapheGTS.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoie;
      N, Nc, hauteurCourbe: integer;
      MaxLoc, MinLoc: double;
   begin { ChercheMonde }
      NbreCourbe := 0;
      for voie := succ(voieT) to voieCalc do
         if voieOrphy[voie].active and (voie <> voieAbscisse) then
            Inc(NbreCourbe);
      if AnalyseurLogique then
         N := NbreCourbe
      else
         N := 1;
      with limiteFenetre do begin
         avecAxeY := ((bottom - top) div 5 > NbreGradMin * hautCarac * N div 4) and
            (AnalyseurLogique or AvecUnAxe);
         avecAxeX := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) div 5 > NbreGraduations * HautCarac * N div 4;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
         MaxGr := 2 * right;
         MinGr := -right;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, Marge);
      Dec(limiteCourbe.bottom, Marge);
      Inc(limiteCourbe.top, Marge);
      if avecAxeY then
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      if avecAxeY and (NbreCourbe > 1) then
         Dec(limiteCourbe.right, LongNombreAxe * largCarac);
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac);
      for voie := voieMin to voieMax do
         with voieOrphy[voie] do begin
            CourbeTop := LimiteCourbe.top;
            CourbeBottom := LimiteCourbe.bottom;
         end;
      if AnalyseurLogique then begin
         hauteurCourbe := ((LimiteCourbe.bottom - LimiteCourbe.top) div N) - marge;
         Nc := 0;
         for voie := succ(voieT) to voieMax do
            with voieOrphy[voie] do
               if active and (voie <> voieAbscisse) then begin
                  CourbeTop := LimiteCourbe.top + Nc * hauteurCourbe + succ(Nc) * marge;
                  CourbeBottom := CourbeTop + hauteurCourbe;
                  Inc(Nc);
               end;
      end;
      with voieOrphy[voieAbscisse] do begin
         MaxLoc := Maxi;
         MinLoc := Mini;
         if loupeAuto and (NbreAff > 16) then
              chercheMinMax(NbreAff, MinLoc, MaxLoc);
         A := (limiteCourbe.right - limiteCourbe.left) / (Maxi - Mini);
         B := limiteCourbe.left - Mini * A;
         setDelta(grandAxeX, mini, maxi);
      end;
      for voie := succ(voieT) to voieMax do
         with voieOrphy[voie] do
            if active and (voie <> voieAbscisse) then begin
               MaxLoc := Maxi;
               MinLoc := Mini;
               if loupeAuto or
                  ((voie>=voieA) and (voie<=voieAnaMax)) then
                      chercheMinMax(NbreAff, MiniLoupe, MaxiLoupe);
               if loupeAuto and (NbreAff > 16) then begin
                   MaxLoc := MaxiLoupe;
                   MinLoc := MiniLoupe;
               end;
               A := (CourbeTop - CourbeBottom) / (MaxLoc - MinLoc);
               B := CourbeBottom - MinLoc * A;
               setDelta(grandAxeY, mini, maxi);
            end;
   end; { ChercheMonde }

   procedure DrawAxis;
   var
      yAxeX: integer;
      xNombreMax: integer;
      PointCourant: double;

      function XcorrectNombre(x: integer): boolean;
      begin
         XcorrectNombre := (x > (limiteCourbe.left + marge)) and
            (x < xNombreMax) and (abs(x - limiteCourbe.left) > marge);
      end;

      function XcorrectGrad(x: integer): boolean;
      begin
         XcorrectGrad := (x > limiteCourbe.left) and (x < limiteCourbe.right);
      end;

      procedure MyLine(x1, y1, x2, y2: integer);
      begin
         canvas.moveTo(x1, y1);
         canvas.lineTo(x2, y2);
      end;

      procedure graduationX(x: integer);
      begin
         if quadrillage then
            MyLine(x, limiteCourbe.top, x, limiteCourbe.bottom)
         else
            MyLine(x, yAxeX, x, yAxeX + marge);
      end;

      procedure initAxeX;
      var
         x1: integer;
      begin with voieOrphy[voieAbscisse] do begin
            PointCourant := PointDebut;
            x1 := XWindow(0);
            canvas.pen.color := colorGr(couleur);
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then begin
               canvas.pen.style := psDash;
               canvas.pen.Width := 1;
               Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
               penDefault;
            end;
            xNombreMax := limiteFenetre.right - largCarac * longNombreAxe div 2;
            with limiteFenetre do begin
               MyLine(left, yAxeX, right, yAxeX);
               { flches }
               MyLine(right - marge * 3, yAxeX + marge, right, yAxeX);
               MyLine(right - marge * 3, yAxeX - marge, right, yAxeX);
            end;
            SetTextAlign(canvas.handle, TA_center + TA_top);
      end end; // initAxeX

      procedure traceAxeX;
      var
         xs: integer;
         NombreAff: string;
      begin with voieOrphy[voieAbscisse] do begin
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            repeat
               xs := Xwindow(PointCourant);
               if XcorrectGrad(xs) then begin
                  graduationX(xs);
                  if XcorrectNombre(xs) then begin
                     if abs(PointCourant) < (Maxi - Mini) / 100 then
                        pointCourant := 0;
                     if axeDateTime then
                        NombreAff := ChaineDuree(pointCourant)
                     else
                        NombreAff := formatAxe(pointCourant / exposant);
                     canvas.TextOut(xs, yAxeX + marge, NombreAff);
                  end;
               end;
               PointCourant := PointCourant + deltaAxe;
            until (PointCourant >= Maxi);
            SetTextAlign(canvas.handle, TA_left + TA_top);
            if axeDateTime then
               NombreAff := nom
            else
               NombreAff := nomAxe(exposant);
            canvas.TextOut(limiteFenetre.left + marge, yAxeX + marge, nombreAff);
         end;
         penDefault;
      end;{traceAxeX}

      procedure traceAxeLogx;
      var
         i, j:  integer;
         xs, xpred, xsucc: integer;
         intermediaire: boolean;
         ecart: integer;
         NombreAff: string;
      begin with voieOrphy[voieAbscisse] do begin
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            Intermediaire := (Maxi - Mini) <= 3;
            Ecart := largCarac * longNombreAxe;
            Xpred := 0;
            Xsucc := 0;
            for i := floor(Mini) to floor(Maxi) do begin
               xs := round(A * i + B);
               if XcorrectGrad(xs) then begin
                  graduationX(xs);
                  if XcorrectNombre(xs) then begin
                     NombreAff := formatUnite(dix(i));
                     canvas.TextOut(xs, yAxeX + marge, NombreAff);
                  end;
               end;
               if intermediaire then begin
                  xpred := xs;
                  xsucc := round(A * succ(i) + B);
               end;
               for j := 2 to 9 do begin
                  xs := round(A * (i + log10(j)) + B);
                  if XcorrectGrad(xs) then
                     if intermediaire then begin
                        graduationX(xs);
                        if XcorrectNombre(xs) and (xs > (xpred + ecart)) and
                           (xs < (xsucc - ecart)) and (xs < (xnombreMax - ecart)) then
                        begin
                           NombreAff := formatAxe(j);
                           canvas.TextOut(xs, yAxeX + marge, NombreAff);
                           xpred := xs;
                        end;
                     end
                     else
                        MyLine(xs, yAxeX, xs, yAxeX + marge);
               end;{for j}
            end; {for i}
         end;
         penDefault;
      end;{ TraceAxeLogx }

      procedure traceAxeY(m: indiceVoie);
      var
         xx, ys: integer;
         nombreAff: string;
         limiteTop: integer;
      begin with voieOrphy[m] do begin
            LimiteTop := CourbeTop + marge;
            if AnalyseurLogique then
               Inc(LimiteTop, hautCarac);
            setTextAlign(canvas.handle, TA_right + TA_top);
            xx := limiteCourbe.left;
            canvas.pen.color := colorGr(couleur);
            PointCourant := PointDebut;
            ys := XWindow(0);
            penDefault;
            if (ys > CourbeTop) and (ys < CourbeBottom) then
               Myline(xx, ys, limiteCourbe.right, ys);
            MyLine(xx, CourbeBottom, xx, CourbeTop);
            { flche }
            MyLine(xx - marge, CourbeTop + 3 * marge, xx, CourbeTop);
            MyLine(xx + marge, CourbeTop + 3 * marge, xx, CourbeTop);
            canvas.font.Color := colorGR(couleur);
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            xx := xx - marge;
            repeat
               ys := Xwindow(PointCourant);
               {coordonne cran de marque}
               if (ys > CourbeTop) and (ys < CourbeBottom) then begin
                  if quadrillage then
                     MyLine(limiteCourbe.left, ys, limiteCourbe.right, ys)
                  else
                     MyLine(limiteCourbe.left, ys, limiteCourbe.left + marge, ys);
                  if (ys > (LimiteTop)) and (ys < (CourbeBottom - marge)) and
                     (abs(ys - yAxeX) > marge) then begin
                     if abs(pointCourant) < (Maxi - Mini) / 1000 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     Canvas.TextOut(xx, ys - marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until (pointCourant >= Maxi);
            canvas.TextOut(xx, courbeTop - marge, nomAxe(exposant));
         end;
         penDefault;
      end;{traceAxeY}

      procedure traceAxeYbis(m: indiceVoie);
      var
         ys, xl: integer;
         nombreAff: string;
      begin with voieOrphy[m] do begin
            canvas.pen.color  := colorGR(couleur);
            canvas.font.color := colorGR(couleur);
            setTextAlign(canvas.handle, TA_left + TA_top);
            xl := limiteCourbe.right;
            PointCourant := PointDebut;
            ys := XWindow(0);
            penDefault;
            Myline(limiteCourbe.left, ys, limiteCourbe.right, ys);
            MyLine(xl, LimiteCourbe.Bottom, xl, LimiteCourbe.Top);
            { flche }
            MyLine(xl - marge, LimiteCourbe.Top + 3 * marge, xl, LimiteCourbe.Top);
            MyLine(xl + marge, LimiteCourbe.Top + 3 * marge, xl, LimiteCourbe.Top);
            xl := xl + marge;
            repeat
               ys := XWindow(PointCourant);
               {coordonne cran de marque}
               MyLine(xl, ys, xl - marge, ys);
               if abs(pointCourant) < (Maxi - Mini) / 1000 then
                  pointCourant := 0;
               NombreAff := formatAxe(pointCourant / exposant);
               Canvas.TextOut(xl, ys - marge, NombreAff);
               pointCourant := pointCourant + deltaAxe;
            until (pointCourant >= Maxi);
            canvas.TextOut(xl, LimiteFenetre.Top + marge, nomAxe(exposant));
         end;
      end; // traceAxeYbis

      procedure zeroX;
      var
         x1: integer;
      begin with voieOrphy[voieAbscisse] do begin
            x1 := XWindow(0);
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
               Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
               penDefault;
            end;
         end;
      end;{zeroX}

      procedure zeroY(m: indiceVoie);
      var
         y1: integer;
      begin with voieOrphy[m] do begin
            y1 := Xwindow(0);
            if (y1 > CourbeTop) and (y1 < CourbeBottom) then begin
               penDefault;
               canvas.pen.color := colorGR(couleur);
               Myline(limiteCourbe.left, y1, limiteCourbe.right, y1);
            end;
         end;
      end;{zeroY}

   var
      voie: indiceVoie;
   begin // DrawAxis
      yAxeX := limiteCourbe.bottom;
      if avecAxeX then begin
         initAxeX;
         if voieOrphy[voieAbscisse].AxeLog then
            traceAxeLogX
         else
            traceAxeX;
         zeroX;
      end;
      if AnalyseurLogique then begin
         for voie := succ(voieMin) to voieMax do
            with voieOrphy[voie] do
               if active and (voie <> voieAbscisse) then
                  if avecAxeY then
                        traceAxeY(voie)
                  else
                     zeroY(voie);
      end
      else begin
         if avecAxeY then begin
            voie := succ(voieMin);
            while (voie < voieMax) and (not (voieOrphy[voie].active) or
                  (voie = voieAbscisse)) do
               Inc(voie);
               traceAxeY(voie);
            if voie < voieMax then begin
               Inc(voie);
               while (voie < voieMax) and (not (voieOrphy[voie].active) or
                     (voie = voieAbscisse)) do
                  Inc(voie);
               if voieOrphy[voie].active then
                  traceAxeYbis(voie);
            end;
         end;
         for voie := succ(voieMin) to voieMax do
            if voieOrphy[voie].active and (voie <> voieAbscisse) then
               zeroY(voie);
      end;
      SetTextAlign(canvas.handle, TA_left + TA_top);
   end; // DrawAxis 

var i : integer;
begin
   with limiteFenetre do
      setHauteurFonte((bottom - top) div 30);
   penDefault;
   canvas.pen.Width := 1;
   canvas.pen.color := colorGr(voieOrphy[voieT].couleur);
   canvas.brush.style := bsClear;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   TraceCourbes;
   for i := 0 to pred(ListeRepere.Count) do
       TraceRepere(ListeRepere[i]);
   ReticuleTrace := False;
end;

procedure TgrapheFourier.Draw;

   procedure ChercheMonde;
   var
      voie: indiceVoie;
      N, Nc, hauteurCourbe: integer;
   begin
      NbreCourbe := 0;
      for voie := voieA to voieAnaMax do
         if voieOrphy[voie].active then
            Inc(NbreCourbe);
      if AnalyseurLogique then
         N := NbreCourbe
      else
         N := 1;
      with limiteFenetre do begin
         avecAxeY := ((bottom - top) div 5 > NbreGradMin * hautCarac * N div 4) and
            (AnalyseurLogique or AvecUnAxe);
         avecAxeX := (right - left) > NbreGradMin * 4 * largCarac;
         grandAxeY := (bottom - top) div 5 > NbreGraduations * HautCarac * N div 4;
         grandAxeX := (right - left) > NbreGraduations * 4 * largCarac;
         MaxGr := 2 * right;
         MinGr := -right;
      end;
      limiteCourbe := limiteFenetre;
      Inc(limiteCourbe.left, Marge);
      Dec(limiteCourbe.bottom, Marge);
      Inc(limiteCourbe.top, Marge);
      if avecAxeY then
         Inc(limiteCourbe.left, LongNombreAxe * largCarac);
      if avecAxeY and (NbreCourbe > 1) then
         Dec(limiteCourbe.right, LongNombreAxe * largCarac);
      if avecAxeX then
         Dec(limiteCourbe.bottom, hautCarac);
      for voie := voieA to voieAnaMax do
         with voieOrphy[voie] do if active then begin
            CourbeTop := LimiteCourbe.top;
            CourbeBottom := LimiteCourbe.bottom;
         end;
      if AnalyseurLogique then begin
         hauteurCourbe := ((LimiteCourbe.bottom - LimiteCourbe.top) div N) - marge;
         Nc := 0;
         for voie := voieA to voieAnaMax do
            with voieOrphy[voie] do if active then begin
                  CourbeTop := LimiteCourbe.top + Nc * hauteurCourbe + succ(Nc) * marge;
                  CourbeBottom := CourbeTop + hauteurCourbe;
                  Inc(Nc);
               end;
      end;
      with voieT do begin
         A := (limiteCourbe.right - limiteCourbe.left)/Maxi;
         B := limiteCourbe.left;
         setDelta(grandAxeX);
      end;
      for voie := voieA to voieAnaMax do
         with voieOrphy[voie] do if active then begin
               decibel := decibelGlb;
               if decibel then begin
                   A := (CourbeTop - CourbeBottom) / gammedB;
                   B := CourbeBottom - (20*log(maxi)-gammedB) * A;
               end
               else begin
                   A := (CourbeTop - CourbeBottom) / Maxi;
                   B := CourbeBottom;
               end;
               setDelta(grandAxeY);
            end;
   end; // ChercheMonde Fourier

   procedure DrawAxis;
   var
      yAxeX: integer;
      xNombreMax: integer;
      PointCourant: double;

      function XcorrectNombre(x: integer): boolean;
      begin
         XcorrectNombre := (x > (limiteCourbe.left + marge)) and
            (x < xNombreMax) and (abs(x - limiteCourbe.left) > marge);
      end;

      function XcorrectGrad(x: integer): boolean;
      begin
         XcorrectGrad := (x > limiteCourbe.left) and (x < limiteCourbe.right);
      end;

      procedure MyLine(x1, y1, x2, y2: integer);
      begin
         canvas.moveTo(x1, y1);
         canvas.lineTo(x2, y2);
      end;

      procedure graduationX(x: integer);
      begin
         if quadrillage then
            MyLine(x, limiteCourbe.top, x, limiteCourbe.bottom)
         else
            MyLine(x, yAxeX, x, yAxeX + marge);
      end;

      procedure initAxeX;
      var
         x1: integer;
      begin with voieT do begin
            PointCourant := PointDebut;
            x1 := XWindow(0);
            canvas.pen.color := colorGr(couleur);
            if (x1 > limiteCourbe.left) and (x1 < limiteCourbe.right) then begin
               canvas.pen.style := psDash;
               canvas.pen.Width := 1;
               Myline(x1, limiteCourbe.bottom, x1, limiteCourbe.top);
               penDefault;
            end;
            xNombreMax := limiteFenetre.right - largCarac * longNombreAxe div 2;
            with limiteFenetre do begin
               MyLine(left, yAxeX, right, yAxeX);
               { flches }
               MyLine(right - marge * 3, yAxeX + marge, right, yAxeX);
               MyLine(right - marge * 3, yAxeX - marge, right, yAxeX);
            end;
            SetTextAlign(canvas.handle, TA_center + TA_top);
      end end; // initAxeX

      procedure traceAxeX;
      var
         xs: integer;
         NombreAff: string;
      begin with voieT do begin
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            repeat
               xs := Xwindow(PointCourant);
               if XcorrectGrad(xs) then begin
                  graduationX(xs);
                  if XcorrectNombre(xs) then begin
                     if abs(PointCourant) < Maxi / 100 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     canvas.TextOut(xs, yAxeX + marge, NombreAff);
                  end;
               end;
               PointCourant := PointCourant + deltaAxe;
            until (PointCourant >= Maxi);
            SetTextAlign(canvas.handle, TA_left + TA_top);
            NombreAff := nomAxe(exposant);
            canvas.TextOut(limiteFenetre.left + marge, yAxeX + marge, nombreAff);
         end;
         penDefault;
      end; // traceAxeX Fourier

      procedure traceAxeY(m: indiceVoie);
      var
         xx, ys: integer;
         nombreAff: string;
         limiteTop: integer;
      begin with voieOrphy[m] do begin
            LimiteTop := CourbeTop + marge;
            if AnalyseurLogique then
               Inc(LimiteTop, hautCarac);
            setTextAlign(canvas.handle, TA_right + TA_top);
            xx := limiteCourbe.left;
            canvas.pen.color := colorGr(couleur);
            PointCourant := PointDebut;
            penDefault;
            MyLine(xx, CourbeBottom, xx, CourbeTop); // axe
            { flche }
            MyLine(xx - marge, CourbeTop + 3 * marge, xx, CourbeTop);
            MyLine(xx + marge, CourbeTop + 3 * marge, xx, CourbeTop);
            canvas.font.Color := colorGR(couleur);
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            xx := xx - marge;
            repeat
               ys := Xwindow(PointCourant);
               {coordonne cran de marque}
               if (ys > CourbeTop) and (ys < CourbeBottom) then begin
                  if quadrillage then
                     MyLine(limiteCourbe.left, ys, limiteCourbe.right, ys)
                  else
                     MyLine(limiteCourbe.left, ys, limiteCourbe.left + marge, ys);
                  if (ys > (LimiteTop)) and (ys < (CourbeBottom - marge)) and
                     (abs(ys - yAxeX) > marge) then begin
                     if abs(pointCourant) < Maxi / 1000 then
                        pointCourant := 0;
                     NombreAff := formatAxe(pointCourant / exposant);
                     Canvas.TextOut(xx, ys - marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until (pointCourant >= Maxi);
            canvas.TextOut(xx, courbeTop - marge, nomAxe(exposant));
         end;
         penDefault;
      end; // traceAxeY Fourier

      procedure traceAxedB(m: indiceVoie);
      var
         xx, ys: integer;
         nombreAff: string;
         limiteTop: integer;
      begin with voieOrphy[m] do begin
            LimiteTop := CourbeTop + marge;
            if AnalyseurLogique then
               Inc(LimiteTop, hautCarac);
            setTextAlign(canvas.handle, TA_right + TA_top);
            xx := limiteCourbe.left;
            canvas.pen.color := colorGr(couleur);
            PointCourant := PointDebut;
            penDefault;
            MyLine(xx, CourbeBottom, xx, CourbeTop);
            { flche }
            MyLine(xx - marge, CourbeTop + 3 * marge, xx, CourbeTop);
            MyLine(xx + marge, CourbeTop + 3 * marge, xx, CourbeTop);
            canvas.font.Color := colorGR(couleur);
            if Quadrillage then begin
               canvas.pen.style := psDot;
               canvas.pen.Width := 1;
            end;
            xx := xx - marge;
            repeat
               ys := Xwindow(PointCourant);
               {coordonne cran de marque}
               if (ys > CourbeTop) and (ys < CourbeBottom) then begin
                  if quadrillage then
                     MyLine(limiteCourbe.left, ys, limiteCourbe.right, ys)
                  else
                     MyLine(limiteCourbe.left, ys, limiteCourbe.left + marge, ys);
                  if (ys > (LimiteTop)) and (ys < (CourbeBottom - marge)) and
                     (abs(ys - yAxeX) > marge) then begin
                     NombreAff := intToStr(round(pointCourant));
                     Canvas.TextOut(xx, ys - marge, NombreAff);
                  end;
               end;
               pointCourant := pointCourant + deltaAxe;
            until (pointCourant >= 20*log(Maxi));
            canvas.TextOut(xx, courbeTop - marge, nom+'(dB)');
         end;
         penDefault;
      end; // traceAxedB Fourier

   var
      voie,voieAxe: indiceVoie;
      maxLoc : double;
   begin // DrawAxis
      yAxeX := limiteCourbe.bottom;
      if avecAxeX then begin
         initAxeX;
         traceAxeX;
      end;
      if AnalyseurLogique then begin
         for voie := voieA to voieAnaMax do
            with voieOrphy[voie] do
               if active then
                  if avecAxeY then traceAxeY(voie)
      end
      else begin
         if avecAxeY then begin
            maxLoc := 0;
            voieAxe := voieA;
            for voie := voieA to voieAnaMax do
                if voieOrphy[voie].active then begin
                   if voieOrphy[voie].maxi>maxLoc then begin
                      maxLoc := voieOrphy[voie].maxi;
                      voieAxe := voie;
                   end;
                end;
            if decibelGlb
               then traceAxedB(voieAxe)
               else traceAxeY(voieAxe);
         end;
      end;
      SetTextAlign(canvas.handle, TA_left + TA_top);
   end; // DrawAxis

begin
   with limiteFenetre do
      setHauteurFonte((bottom - top) div 30);
   penDefault;
   canvas.pen.Width := 1;
   canvas.pen.color := clBlack;
   canvas.brush.style := bsClear;
   setBkMode(canvas.handle, transparent);
   chercheMonde;
   drawAxis;
   TraceCourbes;
end; // grapheFourier.draw

procedure TgrapheGTS.TraceCourbes;

   procedure TraceUneCourbe(voie: integer);

      procedure traceImpulsion;
      var
         i: indiceVecteur;
         xi, y0, y1: integer;
      begin
         penDefault;
         canvas.pen.color := clRed;
         y0 := voieOrphy[voieF].XWindow(0);
         y1 := voieOrphy[voieF].XWindow(voieOrphy[voieF].maxi*2/3);
         for i := 0 to pred(NbreAff) do begin
            xi := voieOrphy[voieAbscisse].XWindow(vecteurTemps[i]);
            canvas.MoveTo(xi, y0);
            canvas.LineTo(xi, y1);
         end;{for i}
      end; // traceImpulsion

      procedure TracePoints;
      var
         i: integer;
         xi, yi: integer;
      begin with voieOrphy[voie] do begin
            canvas.brush.color := colorGr(couleur);
            if (modeAcq in [AcqClavier, AcqPoint]) and (NbreAff < 32) then
               canvas.pen.Width := 3;
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi <> IntSansSignif) and (yi <> IntSansSignif) then
                  TraceMotif(xi, yi, motif);
            end;{i}
            penDefault;
         end;
      end; // tracePoints

      Procedure traceSpiro;
var i,iMax,j,Nbre : integer;
    x1,y1 : Integer;
    x0,y0 : Integer;
    Ecart : double;
    debutH,finH : double;
    valeurMoy : double;
begin with voieOrphy[voie] do begin
     ecart := voieOrphy[voieT].vecteurAff[pred(NbreAff)]-voieOrphy[voieT].vecteurAff[0];
     if (deltaSpiro<ecart/64) or
        (deltaSpiro>ecart/4) then
             deltaSpiro := ecart/8;
     imax := Floor(ecart/deltaSpiro);
     j := 0;
     for i := 0 to imax do begin
         debutH := voieOrphy[voieT].vecteurAff[0]+i*deltaSpiro;
         finH := debutH+deltaSpiro;
         if finH>voieOrphy[voieT].vecteurAff[pred(NbreAff)] then
            finH := voieOrphy[voieT].vecteurAff[pred(NbreAff)];
         valeurMoy := 0;
         Nbre := 0;
         while (j<NbreAff) and (voieOrphy[voieT].vecteurAff[j]<finH) do begin
             valeurMoy := valeurMoy+vecteurAff[j];
             inc(j);
             inc(Nbre);
         end;
         if nbre>0 then valeurMoy := valeurMoy/Nbre;
         y0 := XWindow(0);
         x0 := voieOrphy[voieT].Xwindow(debutH);
         y1 := XWindow(valeurMoy);
         x1 := voieOrphy[voieT].Xwindow(finH);
         canvas.rectangle(x0,y0,x1,y1);
     end;
end end; { traceSpirometrie }

      procedure traceLignes;
      var
         i: integer;
         PointPrecedent: boolean;
         xi, yi: integer;
      begin with voieOrphy[voie] do begin
            PointPrecedent := False;
            for i := 0 to pred(NbreAff) do begin
               xi := voieOrphy[voieAbscisse].XWindow(voieOrphy[voieAbscisse].vecteurAff[i]);
               yi := XWindow(vecteurAff[i]);
               if (xi = IntSansSignif) or (yi = IntSansSignif) then
                  pointPrecedent := False
               else if PointPrecedent then
                  canvas.LineTo(xi, yi)
               else begin
                  canvas.MoveTo(xi, yi);
                  pointPrecedent := True;
               end;
            end;{for i}
         end; {i}
      end; // traceLignes

   begin with voieOrphy[voie] do begin
         canvas.pen.color := colorGr(couleur);
         if trLigne in Trace then
            TraceLignes;
         if trPoint in trace then
            TracePoints;
         if TrSpiro in trace then
            TraceSpiro;
         if (TrCardiaque in trace) and
            (voie = voieF) then
               TraceImpulsion;
      end;
   end; // TraceUneCourbe

var
   voie: indiceVoie;
   yi: integer;
begin
   penDefault;
   if NbreAff > 0 then
      for voie := succ(voieMin) to voieMax do
         if voieOrphy[voie].active and (voie <> voieAbscisse) then
            traceUneCourbe(voie);
   if (modeAcq in [AcqTemps, AcqUnMega]) and (modeSynchro = s_seuil) then begin
      yi := voieOrphy[voieSynchro].XWindow(seuilSynchroReal);
      canvas.pen.style := psDashDot;
      canvas.pen.Width := 1;
      canvas.pen.color := voieOrphy[voieSynchro].couleur;
      canvas.font.color := voieOrphy[voieSynchro].couleur;
      canvas.moveTo(limiteCourbe.left - 16, yi);
      canvas.lineTo(limiteCourbe.right, yi);
      canvas.TextOut(LimiteCourbe.left - 16, yi - 8, Fleche[montant]);
      penDefault;
   end;
end; // TraceCourbes

procedure TgrapheFourier.TraceCourbes;
var decalage,delta,y0 : integer;
    i : integer;
    xi,yi : integer;
    voie: indiceVoie;
begin
    if NbreAff <2 then exit;
    canvas.pen.mode  := pmCopy;
    canvas.pen.style := psSolid;
    delta := 1;
    if trHisto in trace then begin
        WidthHisto := LimiteFenetre.right div (3*NbreAff);
        if impressionEnCours
          then begin
             if WidthHisto>15 then WidthHisto := 15;
          end
          else begin
             if WidthHisto>5 then WidthHisto := 5;
          end;
        if widthHisto<2 then widthHisto := 2;
        canvas.pen.Width := widthHisto;
        delta := widthHisto div 2;
    end;
    if trRaie in trace then begin
       if impressionEnCours
          then canvas.pen.Width := 3
          else canvas.pen.Width := 1;
       delta := (limiteCourbe.Right div (3*nbreAff)) + 1;
   end;
   decalage := 0;
   for voie := voieA to voieAnaMax do
       if voieOrphy[voie].active then begin
          with voieOrphy[voie] do if decibel
               then y0 := courbeBottom
               else y0 := XWindow(0);
          canvas.pen.color := voieOrphy[voie].couleur;
          canvas.brush.color := voieOrphy[voie].couleur;
          for i := 0 to pred(NbreAff) do begin
              with voieT do xi := XWindow(vecteurAff[i])+decalage;
              with voieOrphy[voie] do if decibel
                   then if vecteurAff[i]>maxi*gammeFourier
                      then yi := XWindow(20*log(vecteurAff[i]))
                      else yi := y0
                   else yi := XWindow(vecteurAff[i]);
              if yi>y0 then continue;
              if trHisto in trace then
                 canvas.FillRect(rect(xi-WidthHisto,yi,xi+WidthHisto,y0));
              if trRaie in trace then begin
                  canvas.moveTo(xi,y0);
                  canvas.lineTo(xi,yi);
              end;
          end;
          inc(decalage,delta);
     end;
end; // grapheFourier.TraceCourbes

destructor TgrapheGTS.Destroy;
var
   i: indiceVoie;
begin
   for i := voieMin to VoieMax do
      voieOrphy[i].Free;
   ListeRepere.Free;
   sondor.Free;
   inherited Destroy;
end;

destructor TgrapheFourier.Destroy;
var
   i: indiceVoie;
begin
   for i := voieA to VoieAnaMax do
      voieOrphy[i].Free;
   voieT.free;   
   inherited Destroy;
end;

constructor TgrapheGTS.Create;

   procedure ValeurDefaut;
   begin
      voieAbscisse := voieT;
      Trace := [trLigne];
      quadrillage := False;
      AnalyseurLogique := False;
      Monocoup := True;
      AvecUnAxe := True;
      FrontSynchro := '1';
      montant := True;
      modeAcq := AcqTemps;
      seuilSynchro := 128;
      modeSynchro := s_clavier;
      Magnum.active := False;
      FvoieSynchro := voieA;
      deltaBouge := 2;
      deltatFiltre := 20;
      voieFiltreEntree := voieA;
      voieFiltreSortie := voieA + 1;
      voieFiltrePhase := voieA + 2;
      impressionEnCours := False;
      monochrome := False;
   end;

var
   voie: indiceVoie;
begin
   inherited Create;
   ListeRepere := TListeRepere.Create;
   Magnum  := Tmagnum.Create;
   voieOrphy[voieMagnum] := Magnum;
   DeltaSpiro := 1;
   tension := Tunite.Create;
   tension.nomUnite := 'V';
   BaseDeTemps := TbaseDeTemps.Create;
   voieOrphy[voieT] := BaseDeTemps;
   sondor  := Tsondor.Create;
   voieOrphy[voieSondor] := Sondor;
   frequence := Tfrequence.Create;
   voieOrphy[voieF] := Frequence;
   SortieAnalogique := TsortieAnalogique.Create;
   voieOrphy[voieSA] := SortieAnalogique;
   voieCalculee := TvoieCalculee.Create;
   voieOrphy[voieCalc] := voieCalculee;
   SortiePWM := TsortiePWM.Create;
   EntreeClavier := TentreeClavier.Create;
   voieOrphy[voieClavier] := EntreeClavier;
   for voie := voieA to voieAnaMax do begin
      voieOrphy[voie] := Tcourbe.Create('V' + IntToStr(voie - voieA + 1), 'V', '', voie);
      voieOrphy[voie].setMinMax;
   end;
   voieOrphy[voieA].active := True;
   voieOrphy[voieA].numOrphy := 10;
   voieOrphy[succ(voieA)].active := True;
   voieOrphy[succ(voieA)].numOrphy := 11;
   for voie := voieMin to voieMax do
      with voieOrphy[voie] do
         if couleur = clWindow then
            couleur := clActiveCaption;
   ValeurDefaut;
end;

constructor TgrapheFourier.Create;
var
   voie: indiceVoie;
begin
   inherited Create;
   voieT := TcourbeFourier.create('f','Hz','frquence');
   for voie := voieA to voieAnaMax do
      voieOrphy[voie] := TcourbeFourier.Create('V' + IntToStr(voie - voieA + 1), 'V', '');
   Trace := [trHisto];
end;

constructor Tcourbe.Create(Anom: string; Aunite: string; Asignif: string;
   Anumero: integer);
begin
   inherited Create(Anumero);
   active := False;
   A := 1;
   B := 0;
   NbreMoyenne := 1;
   Mini := -10;
   Maxi := 10;
   nom := Anom;
   NomUnite := Aunite;
   IndexDernier := 0;
   signif := Asignif;
   axeLog := False;
end;

constructor TcourbeFourier.Create(Anom: string; Aunite: string; Asignif: String);
begin
   inherited Create;
   active := False;
   A := 1;
   B := 0;
   Maxi := 10;
   nom := Anom;
   NomUnite := Aunite;
end;

constructor TsortieAnalogique.Create;
var
   i: integer;
begin
   inherited Create('Vsa', 'V', trTensionSA, voieSA);
   DebutSortie[1] := 0;
   FinSortie[1] := maxiHardSA;
   for i := 2 to NbreRampeMax do begin
      DebutSortie[i] := 0;
      FinSortie[i] := 0;
   end;
   nbreRampes := 1;
   SAi := '0';
   deltatSA := 1000;
   nbrePulse := 2;
   largeurPulse := 2;
   distancePulse := 10;
   debutPulse := 2;
   numOrphy := -1;
end;

constructor TentreeClavier.Create;
begin
   inherited Create('X', '', '', voieClavier);
end;

constructor TvoieCalculee.Create;
begin
   inherited Create('X', '', '', voieCalc);
   avecCapteur := True;
end;

constructor TsortiePWM.Create;
begin
   inherited Create('Vpwm', 'V', trTensionPWM, voieSA);
   PWMi := '0';
   Ffrequence := 1000;
   Fduty100 := 50;
   setPeriode;
   numOrphy := -1;
end;

procedure TsortiePWM.setFrequence(Avalue: integer);
begin
   if (Avalue > 50) and (Avalue < 50000) then
   begin
      Ffrequence := Avalue;
      setPeriode;
   end;
end;

procedure TsortiePWM.setDuty(Avalue: integer);
begin
   if (Avalue > 0) and (Avalue < 100) then
   begin
      Fduty100 := Avalue;
      setPeriode;
   end;
end;

constructor Tfrequence.Create;
begin
   inherited Create('f', 'Hz', trFrequence, voieF);
   Mini  := 100;
   MiniF := Mini;
   Maxi  := 10000;
   MaxiF := Maxi;
   AxeLog := False;
   seuil := 128;
   FreqEA11 := False;
   numVoie := 0;
   numOrphy := -1;
end;

function Tfrequence.Commande: string;
begin
   if FreqEA11
      then if isGTI2
         then Result := 'WAUTO ' + IntToStr(numVoie)+' '+IntToStr(seuil)
         else Result := 'WAUTO ' + IntToStr(seuil)
      else Result := 'WPER ' + IntToStr(numVoie);
end;

function Tfrequence.Stop: string;
begin
   if FreqEA11
      then if isGTI2
         then Result := 'WAUTO 12'
         else Result := 'WAUTO 0'
      else Result := 'WPER 4'
end;

constructor Tmagnum.Create;
begin
   inherited Create('z', 'm', trMagnum, voieMagnum);
   modeAngulaire := False;
   modeRadian := False;
   EFi := 0;
   EBi := 5;
   TraitMagnumE := 0.0018;
   OscilMin := 8;
   DureeOscillation := 2;
   increment := traitMagnum;
   isCodeurInc := False;
   numOrphy := -1;
end;

constructor Tsondor.Create;
begin
   inherited Create('dt', 's', trSondor, voieSondor);
   EFi  := 0;
   SBi  := 8;
   EAi  := 8;
   mini := 0;
   maxi := 0.1;
   setMinMax;
end;

procedure Tmagnum.RecopieM(N: integer);
var
   vW, maxW, trait: double;
   k: integer;
   z, maxz, minz, zeroz: integer;
begin
   if Fchute then begin
      if Negatif then begin
         trait := -2 * traitMagnumE;
         maxi  := 0;
         mini  := (N + 2) * trait;
      end
      else begin
         trait := +2 * traitMagnumE;
         maxi  := (N + 2) * trait;
         mini  := 0;
      end;
      for k := 0 to pred(N) do
          vecteurAff[k] := k * trait;
   end
   else begin
      z := vecteurCourant[0];
      maxz := z;
      minz := z;
      for k := 1 to pred(N) do
      begin
         z := vecteurCourant[k];
         if z > maxz then
            maxz := z;
         if z < minz then
            minz := z;
      end;
      zeroz := (maxz + minz) div 2;
      maxW  := 0;
      for k := 0 to pred(N) do begin
         vecteurAff[k] := (vecteurCourant[k] - zeroz) * traitMagnumE;
         if (k < (N - 1)) and (grapheGTS.voieOrphy[voieT].vecteurAff[k] >
            grapheGTS.voieOrphy[voieT].vecteurAff[succ(k)]) then
            // voir fichier magnum_gti2 de Thierry Dulaurans
            vecteurAff[k] := sansSignif;
         vW := vecteurAff[k];
         if abs(vW) > maxW then
            maxW := abs(vW);
      end;
      if maxW > 10 * TraitMagnumE then begin
         maxi := maxW * 1.05;
         mini := -maxi;
      end
      else begin
         maxi := 11 * traitMagnumE;
         mini := -maxi;
      end;
   end;
end;

constructor TbaseDeTemps.Create;
begin
   inherited Create('t', 's', trDuree, voieT);
   Mini := 0;
   Maxi := 1;
   numOrphy := -1;
end;

destructor Tcourbe.Destroy;
begin
   inherited Destroy;
end;

procedure Tcourbe.SetDelta(grandAxe: boolean; min, max: double);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   AbsMax, AbsMin, largeur, z: double;
   Exponent, index: integer;
   N: integer;
begin
   if Maxi < Mini then begin
      z := Maxi;
      Maxi := Mini;
      Mini := z;
   end;
   VerifMinMaxDouble(min, max);
   if Maxi = Mini then
      Maxi := Mini + 1;
   if Max = Min then
      Max := Min + 1;
   AbsMax := abs(max);
   AbsMin := abs(min);
   if AbsMax < AbsMin then
      AbsMax := AbsMin;
   axeDateTime := (nomUnite = 's') and (AbsMax > QuartdHeure);
   if axeDateTime then begin
      NbreChiffre := longNombreAxe;
      DeltaAxe := (Max - Min) / 6;
      if DeltaAxe > UnJour then
         DeltaAxe := ceil(DeltaAxe / UnJour) * UnJour
      else if DeltaAxe > UneHeure then begin
         N := round(DeltaAxe / UneHeure);
         case N of
            1, 2, 3, 4: ;
            5: N := 6;
            6: ;
            7: N := 8;
            8: ;
            9, 10, 11: N := 12;
            12: ;
            13, 14, 15, 16, 17: N := 12;
            18, 19, 20, 21, 22, 23, 24: N := 24;
         end;
         DeltaAxe := N * UneHeure;
      end
      else if DeltaAxe > QuartdHeure then
         DeltaAxe := ceil(DeltaAxe / QuartdHeure) * QuartdHeure
      else
         DeltaAxe := ceil(DeltaAxe / UneMinute) * UneMinute;
      PointDebut := 0;
      Exposant := 1;
      exit;
   end;
   Exponent := floor(Log10(Max - Min));
   largeur := (max - min) * dix(-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := ceil(Log10(AbsMax)) - floor(Log10(DeltaAxe));
   PointDebut := ceil(min / deltaAxe) * deltaAxe;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
   { la division euclidienne des informaticiens <> div des maths ! }
   Exposant := Dix(3 * Index);
   N := ceil(Log10(AbsMax / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) or axeLog then
      NbreChiffre := longNombreAxe;
end;

procedure TcourbeFourier.SetDelta(grandAxe: boolean);
{ Exposant=puissance de 10 multiple de 3 de manire  avoir entre
NbreGraduations (10) et N/2.5 (4) graduations sur l'axes.
ou NbreGradMin (5) et N/2.5 (2) graduations sur l'axes.
Delta est l'intervalle entre les graduations qui commencent  PremiereGraduation }
const
   coeff: array[0..5] of double = (0.1, 0.2, 0.5, 1, 2, 5);
var
   largeur: double;
   Exponent, index: integer;
   N: integer;
begin
   if decibel then begin
      pointDebut := 20*log(maxi)-gammedB;
      deltaAxe := 10; // dB
      pointDebut := 10*ceil(pointDebut/10);
      NbreChiffre := 2;
      exposant := 1;
      exit;
   end;
   Exponent := floor(Log10(Maxi));
   largeur := maxi * dix(-exponent);
   index := 0;
   if grandAxe then
      while ((largeur / coeff[index]) >= NbreGraduations) and (index < 5) do
         Inc(index)
   else
      while ((largeur / coeff[index]) >= NbreGradMin) and (index < 5) do
         Inc(index);
   DeltaAxe := coeff[index] * dix(exponent);
   NbreChiffre := ceil(Log10(maxi)) - floor(Log10(DeltaAxe));
   PointDebut := 0;
   Index := Exponent div 3;
   if ((Exponent mod 3) < 0) then
      Dec(index);
// la division euclidienne des informaticiens <> div des maths ! 
   Exposant := Dix(3 * Index);
   N := ceil(Log10( (Maxi) / Exposant));
   if N > NbreChiffre then
      NbreChiffre := N;
   if NbreChiffre < 3 then
      NbreChiffre := 3;
   if (NbreChiffre > longNombreAxe) then
      NbreChiffre := longNombreAxe;
end;

function Tcourbe.FormatAxe(x: double): string;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

function TcourbeFourier.FormatAxe(x: double): string;
begin
   FormatAxe := FloatToStrF(x, ffGeneral, NbreChiffre, 0);
end;

procedure TgrapheGts.Tri;
 { Tri Shell Meyer-Baudoin p 456 }
 { origine vecteur en 0 }
var
   cle: double;
   i, j: integer;
   v: indiceVoie;
   SauvePoint: array[indiceVoie] of double;
begin
   if modeAcq = AcqTemps then
      exit;
   if not voieOrphy[voieF].active then
      exit;
   i := 1;
   while (i < NbreAff) do begin
      for v := voieT to VoieMax do
         sauvePoint[v] := voieOrphy[v].vecteurAff[i];
      cle := voieOrphy[voieF].vecteurAff[i];
      j := pred(i);
      while (j >= 0) and (voieOrphy[voieF].vecteurAff[j] > cle) do begin
         for v := voieT to VoieMax do
            voieOrphy[v].vecteurAff[succ(j)] := voieOrphy[v].vecteurAff[j];
         Dec(j);
      end;
      for v := voieT to VoieMax do
         voieOrphy[v].vecteurAff[succ(j)] := sauvePoint[v];
      Inc(i);
   end;
end;{tri}

function TsortieAnalogique.ValeurVolt(V: integer): double;
begin
   Result := V * 5.0 / maxiHardSA;
   if SAi = '1' then
      Result := 2 * Result - 5.0;
end;

function TsortieAnalogique.ValeurReal(V: integer): double;
begin
   Result := valeurVolt(V);
   if expression <> '' then
      if parser <> nil then begin
         Parser.V := Result;
         Result := Parser.Value;
      end
      else
         Result := sansSignif;
end;

function Tmagnum.ValeurReal(V: integer): double;
begin
   if IsCodeurInc then
      Result := (V - zeroMagnum) * increment
   else
      Result := V * traitMagnumE;
end;

function Tmagnum.codeMagnum: string;
begin
   if Magnum.chute then
      Result := 'WDEF ' + IntToStr(EFi)
   else
      Result := 'WCODINC ' + IntToStr(EFi) + ' ' + IntToStr(EBi);
end;

function Tsondor.commande(mode: TmodeAcq): string;
begin
   case mode of
      AcqClavier, AcqPoint, AcqSortieAna: Result :=
            'WSON ' + IntToStr(EFi) + ' ' + IntToStr(SBi);
      AcqTemps, AcqUnMega: Result := 'WSONDOR ' + IntToStr(EFi) + ' ' + IntToStr(SBi);
      else
         Result := '';
   end;
end;

function Tmagnum.stopMagnum: string;
begin
   if isCodeurInc then
      if CodeurActif then begin
         Result := 'WPOSITION'; { dsactive }
         codeurActif := False;
      end
      else
   else if Magnum.chute then
      Result := 'WSDEF'
   else
      Result := 'WSCODINC';
end;

function Tmagnum.codeCodeurInc: string;
begin
   Result := 'WPOSITION ' + IntToStr(EFi) + ' ' + IntToStr(EBi);
   codeurActif := True;
end;

procedure TgrapheGTS.SetOrigineTemps;
var
   seuil: double;
   i, j, imax: integer;
   frontAtteint: boolean;
   y1, y2, t1, t2, t0: double;
begin
   if BaseDeTemps.NbrePreAcq = 0 then begin
      if (modeSynchro = s_seuil) and
         (abs(seuilSynchroReal) < 0.2) then begin // 2*78 mV = 0.156 V mini
         // on considre que c'est zro et on met t=0 pour V=0
         y1 := voieOrphy[FvoieSynchro].vecteurAff[0];
         y2 := voieOrphy[FvoieSynchro].vecteurAff[1];
         t1 := voieOrphy[voieT].vecteurAff[0];
         t2 := voieOrphy[voieT].vecteurAff[1];
         try
            t0 := t1 - y1 * (t2 - t1) / (y2 - y1);
         except
            exit;
         end;
         if (t0 > 0) or (abs(t0) > 3 * baseDeTemps.deltat) then
            exit;
         with BaseDeTemps do
            for j := 0 to MaxVecteur do
               vecteurAff[j] := vecteurAff[j] - t0;
      end;
   end
   else begin                                               
      seuil := seuilSynchroReal;
      i := 0;
      imax := BaseDeTemps.NbrePreAcq;
      with voieOrphy[FvoieSynchro] do begin
         // on attend d'tre en dessous
         repeat
            Inc(i);
            if montant then
               frontAtteint := vecteurAff[i] < seuil
            else
               frontAtteint := vecteurAff[i] > seuil;
         until (i >= imax) or frontAtteint;
         // on attend la traverse   
         repeat
            Inc(i);
            if montant then
               frontAtteint := vecteurAff[i] > seuil
            else
               frontAtteint := vecteurAff[i] < seuil;
         until (i >= imax) or frontAtteint;
      end;
      with BaseDeTemps do
         for j := 0 to MaxVecteur do
            vecteurAff[j] := deltat * (j - i);
   end;
end;

procedure TgrapheGTS.CourantVersAff;
var
   voie: indiceVoie;
   CompteurVoieAna: integer;

{
Procedure DecaleSinc(voie : indiceVoie);
var i,k : integer;
    alphaLoc: double;
    kmax : integer;
    table : array[-maxInterpSinc..+maxInterpSinc] of double;
    vecteurLoc : vecteur;
begin  with voieOrphy[voie] do begin
     AlphaLoc := CoeffInterp * CompteurVoieAna;
     for k := -maxInterpSinc to maxInterpSinc do
         table[k] := sinc(alphaLoc+k);
     for i := 0 to pred(nbreCourant) do
         vecteurLoc[i] := valeurReal(vecteurCourant[i]);
     for i := 0 to pred(maxInterpSinc) do begin
         vecteurAff[i] := 0;
         for k := -i to +i do
             vecteurAff[i] := vecteurAff[i]+vecteurLoc[i+k]*table[k];
     end;
     for i := maxInterpSinc to pred(nbreCourant-maxInterpSinc) do begin
         vecteurAff[i] := 0;
         for k := -maxInterpSinc to maxInterpSinc do
             vecteurAff[i] := vecteurAff[i]+vecteurLoc[i+k]*table[k];
     end;
     for i := (nbreCourant-maxInterpSinc) to pred(NbreCourant) do begin
         vecteurAff[i] := 0;
         kmax := NbreCourant-i-1;
         for k := -kmax to kmax do
             vecteurAff[i] := vecteurAff[i]+vecteurLoc[i+k]*table[k];
     end;
     Inc(CompteurVoieAna);
end end; // DecaleSinc
}

   procedure Decale(voie: indiceVoie);
   var
      j: integer;
      alphaLoc: double;
   begin with voieOrphy[voie] do begin
         AlphaLoc := CoeffInterp * CompteurVoieAna;
         vecteurAff[0] := valeurReal(vecteurCourant[0]) * (1 + alphaLoc) -
            valeurReal(vecteurCourant[1]) * alphaLoc;
         for j := 1 to pred(nbreCourant) do
            vecteurAff[j] := valeurReal(vecteurCourant[pred(j)]) * alphaLoc +
               valeurReal(vecteurCourant[j]) * (1 - alphaLoc);
         Inc(CompteurVoieAna);
   end end;

   procedure recopieVoieAna(voie: indiceVoie);
   var
      j: integer;
   begin
      with voieOrphy[voie] do
         for j := 0 to pred(nbreCourant) do
            vecteurAff[j] := valeurReal(vecteurCourant[j]);
      Inc(CompteurVoieAna);
   end;

   procedure AjoutePremierPoint(voie: indiceVoie);
   var
      j: integer;
      dt, pente: double;
   begin with voieOrphy[voie] do begin
         pente := (vecteurAff[1] - vecteurAff[0]) / baseDeTemps.deltat;
         with voieOrphy[voieT] do
            dt := vecteurAff[1] - vecteurAff[0];
         vecteurAff[0] := vecteurAff[1] - pente * dt;
         for j := 1 to pred(nbreCourant) do
            vecteurAff[j] := vecteurAff[pred(j)];
      end;
   end;

   procedure RecaleZero;
   var
      j: integer;
      dt, pente: double;
   begin with voieOrphy[voieSynchro] do begin
         pente := (vecteurAff[1] - vecteurAff[0]) / baseDeTemps.deltat;
         with voieOrphy[voieT] do
            dt := vecteurAff[1] - vecteurAff[0];
         vecteurAff[0] := vecteurAff[1] - pente * dt;
         for j := 1 to pred(nbreCourant) do
            vecteurAff[j] := vecteurAff[pred(j)];
      end;
   end;

   procedure AffecteUnMega;
   var
      j: integer;
{
    CoeffCorrel,TrameRef,NbreMax : integer;
    IsCorrel : array[0..40,0..40] of boolean;
    NbreCorrel : array[0..40] of byte;
}
   begin
{
   for j := 0 to pred(NbreTrame) do begin
       NbreCorrel[j] := 0;
       IsCorrel[j,j] := true;
       for k := succ(j) to pred(NbreTrame) do begin
           CoeffCorrel := 0;
           for i := 0 to 16 do with voieOrphy[voieA] do
               CoeffCorrel := CoeffCorrel+abs(vecteurCourant[i+j]-vecteurCourant[i+k]);
           IsCorrel[j,k] := CoeffCorrel<1;
           if IsCorrel[j,k] then inc(NbreCorrel[j]);
       end;
   end;
   for j := 0 to pred(NbreTrame) do
       for k := 0 to pred(j) do
           IsCorrel[j,k] := IsCorrel[k,j];
   NbreMax := NbreCorrel[0];
   TrameRef := 0;
   for j := 1 to pred(NbreTrame) do begin
       if NbreCorrel[j]>NbreMax then begin
          NbreMax := NbreCorrel[j];
          TrameRef := j;
       end;
   end;
   if NbreMax<NbreTrame then
      for j := 0 to pred(nbreTrame) do if not IsCorrel[j,TrameRef] then begin
          for i := 0 to pred(NbreParVoieParTrame) do with voieOrphy[voieA] do begin
              with voieOrphy[voieA] do vecteurAff[i*NbreParVoieParTrame+j] := sansSignif;
              with voieOrphy[voieB] do vecteurAff[i*NbreParVoieParTrame+j] := sansSignif;
          end;
      end;
}
      for j := 0 to pred(nbreInter) do begin
         with voieOrphy[voieA] do
            vecteurAff[j] := valeurReal(vecteurCourant[j]);
         with voieOrphy[succ(voieA)] do
            vecteurAff[j] := valeurReal(vecteurCourant[j]);
      end;
      for j := 0 to pred(decaleMega) do
         with voieOrphy[succ(voieA)] do
            vecteurAff[j] := sansSignif;
   end;

begin
   if voieOrphy[voieMagnum].active then
      magnum.RecopieM(NbreCourant);
   compteurVoieAna := 0;
   if modeAcq=AcqUnMega
      then affecteUnMega
      else
         for voie := voieA to VoieAnaMax do
            if voieOrphy[voie].active then
               if compteurVoieAna=0
                  then recopieVoieAna(voie)
                  else decale(voie);
   if recalageZero and (modeAcq = AcqTemps) and (ModeSynchro = s_seuil) and
      (voieOrphy[voieSynchro].vecteurAff[0] <> 0) then
   begin
      RecaleZero;
      for voie := voieA to VoieAnaMax do
         if (voie <> voieSynchro) and voieOrphy[voie].active then
            AjoutePremierPoint(voie);
   end;
   if voieCalculee.active then
      voieCalculee.recopieC(NbreCourant);
   NbreAff := NbreCourant;
end; // CourantVersAff

function TsortieAnalogique.GetNomVoieOrphy: string;
begin
   Result := 'SA' + SAi;
end;

function Tsondor.GetNomVoieOrphy: string;
begin
   Result := 'Sondor';
end;

function TentreeClavier.GetNomVoieOrphy: string;
begin
   Result := 'Clavier'
end;

function TvoieCalculee.GetNomVoieOrphy: string;
begin
   Result := 'Calcul'
end;

function TsortiePWM.GetNomVoieOrphy: string;
begin
   Result := 'PWM' + PWMi;
end;

function Tfrequence.GetNomVoieOrphy: string;
begin
   if FreqEA11 then
      Result := 'EAD1'
   else
      Result := 'EF' + IntToStr(numvoie);
end;

{
Procedure TsortieAnalogique.CalcVecteur;
var i,N : integer;
    marge : double;
begin
      N := finSortie-debutSortie;
      for i := 0 to N do if inverse
          then vecteurCourant[i] := finSortie-i
          else vecteurCourant[i] := i+debutSortie;
      maxi := valeurReal(vecteurCourant[0]);
      mini := maxi;
      for i := 0 to N do begin
          vecteurAff[i] := valeurReal(vecteurCourant[i]);
          if vecteurAff[i]>maxi
            then maxi := vecteurAff[i]
            else if vecteurAff[i]<mini
                then mini := vecteurAff[i]
      end;
      marge := (maxi-mini)/16;
      if mini>=0 then
         if mini<marge
            then mini := 0
            else mini := mini-marge;
      maxi := maxi+marge;
end;
}

procedure TsortieAnalogique.CalcVecteur;
var
   i, idebut, j, k, pas, N: integer;
   marge: double;
begin
   NbreInterSA := 0;
   if NbreRampes = 0 then begin
      NbreRampes := 1;
      finSortie[1] := 128;
      debutSortie[1] := 0;
   end;
   for j := 1 to NbreRampes do begin
      if finSortie[j] > debutSortie[j] then
         pas := 1
      else
         pas := -1;
      N := abs(finSortie[j] - debutSortie[j]);
      if N < 1 then
         N := 1;
      if (j = 1) or (debutSortie[j] <> finSortie[j - 1]) then
         idebut := 0
      else
         idebut := 1;
      for i := idebut to N do begin
         k := debutSortie[j] + i * pas;
         vecteurAff[NbreInterSA] := valeurReal(k);
         vecteurCourant[NbreInterSA] := k;
         Inc(NbreInterSA);
      end; // for i
   end; // for j
   maxi := vecteurAff[0];
   mini := maxi;
   for i := 1 to NbreInterSA do begin
      if vecteurAff[i] > maxi then
         maxi := vecteurAff[i]
      else if vecteurAff[i] < mini then
         mini := vecteurAff[i];
   end;
   marge := (maxi - mini) / 16;
   if mini >= 0 then
      if mini < marge then
         mini := 0
      else
         mini := mini - marge;
   maxi := maxi + marge;
end;

procedure TsortieAnalogique.CalcVecteurNerf(NbrePoints: integer);
var
   j, t, tmod, tdiv: integer;
   debutReal, finReal: double;
begin
   debutReal := valeurReal(debutSortie[1]);
   finReal := valeurReal(finSortie[1]);
   for j := 0 to NbrePoints do begin
      t := round(baseDeTemps.deltat * j / deltatWgene[Base32Micro]) - debutPulse;
      // t en numro par rapport au dbut, en unite baseDeTemps SA
      if t <= 0 then
         vecteurAff[j] := debutReal
      else begin
         tmod := t mod distancePulse;
         tdiv := t div distancePulse;
         if (tmod > largeurPulse) or (tdiv >= nbrePulse) then
            vecteurAff[j] := debutReal
         else
            vecteurAff[j] := finReal;
      end;
   end;
   maxi := 5;
   if SAi = '0' then
      mini := 0
   else
      mini := -5;
   NbreInterSA := NbrePoints;
end;

procedure TsortieAnalogique.Load(Avoie: string; fichier: tMemIniFile);
var
   i: integer;
   SAi0 : integer;
begin
   inherited load(Avoie, fichier);
   with fichier do begin
      debutSortie[1] := ReadInteger(Avoie, 'Debut', 50);
      finSortie[1] := ReadInteger(Avoie, 'Fin', 250);
      NbreRampes := ReadInteger(Avoie, 'Nrampes', 1);
      for i := 1 to NbreRampeMax do begin
         debutSortie[i] := ReadInteger(Avoie, 'Debut' + IntToStr(i), debutSortie[i]);
         finSortie[i] := ReadInteger(Avoie, 'Fin' + IntToStr(i), finSortie[i]);
      end;
      SAi0 := ReadInteger(Avoie, 'Code',0);
      SAi := ansichar(SAi0+ord('0'));
      DeltatSA := ReadInteger(Avoie, 'Deltat', DeltatSA);
      nbrePulse := ReadInteger(Avoie, 'NbrePulse', 2);
      largeurPulse := ReadInteger(Avoie, 'LargPulse', 2);
      distancePulse := ReadInteger(Avoie, 'DistPulse', 10);
      debutPulse := ReadInteger(Avoie, 'DebutPulse', 2);
   end;
   CalcVecteur;
end;

procedure TvoieNonEA.setMinMax;
var
   maxmax: double;
   exponent: integer;
begin
   if Mini >= Maxi then
      Maxi := Mini + 1;
   MaxMax := abs(maxi);
   if MaxMax < abs(mini) then
      MaxMax := abs(mini);
   Exponent := floor(Log10(MaxMax));
   if (Exponent mod 3) < 0 then
      Exponent := (Exponent div 3) - 1
   else
      Exponent := Exponent div 3;
   // la division euclidienne des informaticiens <> div des maths !
   setExposant(maxmax, Exponent);
end;

procedure TsortiePWM.Load(Avoie: string; fichier: tMemIniFile);
var pwmi0 : integer;
begin
   inherited load(Avoie, fichier);
   with fichier do begin
      Ffrequence := ReadInteger(Avoie, 'Frequence', 1000);
      Fduty100 := ReadInteger(Avoie, 'Duty', 50);
      PWMi0 := ReadInteger(Avoie, 'Code', 0);
      PWMi := ansichar(pwmi0+ord('0'));
   end;
end;

procedure Tmagnum.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   with fichier do
   begin
      modeAngulaire := ReadBool(Avoie, 'angle', False);
      pasEntier := ReadBool(Avoie, 'pasEntier', False);
      modeRadian := ReadBool(Avoie, 'radian', False);
      arretAuto := ReadBool(Avoie, 'arret', False);
      isCodeurInc := ReadBool(Avoie, 'codinc', False);
      increment := ReadFloat(Avoie, 'increment', TraitMagnum);
      traitMagnumE := ReadFloat(Avoie, 'taille', TraitMagnum);
      OscilMin := ReadInteger(Avoie, 'OscilMin', oscilMin);
      Chute := ReadBool(Avoie, 'Chute', False);
      Negatif := ReadBool(Avoie, 'Negatif', True);
      Montant := ReadBool(Avoie, 'Montant', False);
      Synchro := ReadBool(Avoie, 'Synchro', false);
      DureeOscillation := ReadFloat(Avoie, 'DureeOscil', dureeOscillation);
      EFi := ReadInteger(Avoie, 'Front', 0);
      EBi := ReadInteger(Avoie, 'EBi', 5);
   end;
end;

procedure TsortieAnalogique.Store(Avoie: string; fichier: tMemIniFile);
var
   i: integer;
begin
   inherited store(Avoie, fichier);
   with fichier do begin
      for i := 1 to NbreRampeMax do begin
         WriteInteger(Avoie, 'Debut' + IntToStr(i), debutSortie[i]);
         WriteInteger(Avoie, 'Fin' + IntToStr(i), finSortie[i]);
      end;
      WriteInteger(Avoie, 'Debut', debutSortie[1]);
      WriteInteger(Avoie, 'Nrampes', NbreRampes);
      WriteInteger(Avoie, 'Fin', finSortie[1]);
      WriteInteger(Avoie, 'Code', ord(SAi)-ord('0'));
      WriteInteger(Avoie, 'Deltat', DeltatSA);
      WriteInteger(Avoie, 'NbrePulse', nbrePulse);
      WriteInteger(Avoie, 'LargPulse', largeurPulse);
      WriteInteger(Avoie, 'DistPulse', distancePulse);
      WriteInteger(Avoie, 'DebutPulse', debutPulse);
   end;
end;

procedure TsortiePWM.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do begin
      WriteInteger(Avoie, 'Frequence', Ffrequence);
      WriteInteger(Avoie, 'Duty', Fduty100);
      WriteString(Avoie, 'Code', string(PWMi));
   end;
end;

procedure Tmagnum.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do
   begin
      WriteBool(Avoie, 'radian', FModeRadian);
      WriteBool(Avoie, 'pasEntier', pasEntier);
      WriteBool(Avoie, 'angle', FModeAngulaire);
      WriteBool(Avoie, 'arret', arretAuto);
      WriteBool(Avoie, 'Chute', FChute);
      WriteBool(Avoie, 'Negatif', Negatif);
      WriteBool(Avoie, 'codinc', IsCodeurInc);
      WriteFloat(Avoie, 'Taille', traitMagnumE);
      WriteFloat(Avoie, 'Increment', increment);
      WriteInteger(Avoie, 'OscilMin', oscilMin);
      WriteFloat(Avoie, 'DureeOscil', DureeOscillation);
      writeInteger(Avoie, 'Front', EFi);
      writeInteger(Avoie, 'EBi', EBi);
      writeBool(Avoie, 'Montant', Montant);
      WriteBool(Avoie, 'Synchro', Synchro);
   end;
end;

procedure TbaseDeTemps.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   Maxi := fichier.ReadFloat(Avoie, 'Dure', 1.0);
end;

procedure TbaseDeTemps.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   with fichier do
      WriteFloat(Avoie, 'Dure', Maxi);
end;

procedure Tfrequence.Load(Avoie: string; fichier: tMemIniFile);
begin
   inherited load(Avoie, fichier);
   seuil := fichier.ReadInteger(Avoie, 'Seuil', 128);
   if isGTI2 then begin
          if seuil<130 then seuil := 2048;
          if seuil>4000 then seuil := 2048;
       end
       else begin
          if seuil<16 then seuil := 128;
          if seuil>240 then seuil := 128;
       end;
   AxeLog := fichier.readBool(Avoie, 'Log', False);
   MiniF := fichier.ReadFloat(Avoie, 'miniF', 100);
   MiniF := abs(MiniF);
   MaxiF := fichier.ReadFloat(Avoie, 'maxiF', 10000);
   MaxiF := abs(MaxiF);
   Mini := MiniF;
   Maxi := MaxiF;
   setMinMax;
   FreqEA11 := fichier.ReadBool(Avoie, 'EA', True);
   numVoie  := fichier.ReadInteger(Avoie, 'Voie', 0);
end;

procedure Tfrequence.Store(Avoie: string; fichier: tMemIniFile);
begin
   inherited store(Avoie, fichier);
   fichier.WriteInteger(Avoie, 'Seuil', seuil);
   fichier.WriteBool(Avoie, 'Log', AxeLog);
   fichier.WriteBool(Avoie, 'EA', FreqEA11);
   fichier.WriteInteger(Avoie, 'Voie', numVoie);
   fichier.WriteFloat(Avoie, 'miniF', miniF);
   fichier.WriteFloat(Avoie, 'maxiF', maxiF);
end;

function Tfrequence.CalibreVoie: string;
begin
   Result := formatUnite(maxiF);
end;

function Tsondor.CalibreVoie: string;
begin
   Result := ' s';
end;

function Tmagnum.CalibreVoie: string;
begin
   Result := '';
end;

function TvoieCalculee.CalibreVoie: string;
begin
   Result := '';
end;

function TsortieAnalogique.CalibreVoie: string;
begin
   if SAi = '0' then
      Result := '0-5 V';
   if SAi = '1' then
      Result := '5V';
end;

function TgrapheGTS.getNumSynchro: integer;
begin
   if isGTI2 then
      Result := voieOrphy[FvoieSynchro].numOrphy
   else
      Result := NumSynchroGTS2;
end;

procedure TgrapheGTS.setVoieSynchro(avoie: indiceVoie);
begin
   if isGTI2 then
   begin
      FvoieSynchro := avoie;
      voieOrphy[FvoieSynchro].active := True;
   end;
end;

procedure TgrapheGTS.VerifSynchro;
var
   Acourbe: Tcourbe;
begin
   if (modeAcq = AcqTemps) and (modeSynchro = s_seuil) and (FvoieSynchro > voieA) then
   begin
      Acourbe := VoieOrphy[voieA];
      VoieOrphy[voieA] := VoieOrphy[FvoieSynchro];
      VoieOrphy[FvoieSynchro] := Acourbe;
      VoieOrphy[FvoieSynchro].LabelVoie.tag := FvoieSynchro;
      VoieOrphy[FvoieSynchro].GBVoie.tag := FvoieSynchro;
      VoieOrphy[voieA].LabelVoie.tag := voieA;
      VoieOrphy[voieA].GBVoie.tag := voieA;
      FvoieSynchro := voieA;// voie de synchro doit tre en premier
      VoieOrphy[FvoieSynchro].active := true;// voie de synchro doit tre active
   end;
end;

function TgrapheGTS.VerifSondor: boolean;
var
   voie: indiceVoie;
begin
   Result := False;
   if (modeAcq = AcqTemps) and sondor.active then begin
      modeSynchro := s_sondor;
      for voie := voieA to voieAnaMax do
         if voieOrphy[voie].active and (voieOrphy[voie].numOrphy = sondor.EAi) then
            exit;
      voieOrphy[voieA].active := True;
      voieOrphy[voieA].numOrphy := sondor.EAi;
      Result := True;
   end;
end;

function TgrapheGTS.getVoieSynchro: indiceVoie;

   function VoieSynchroOK(voie: indiceVoie): boolean;
   begin
      Result := voieOrphy[voie].active;
      if isGTI2 then
         Result := Result and (voieOrphy[voie].numOrphy >= 0)
      else
         Result := Result and (voieOrphy[voie].numOrphy = numSynchroGTS2);
   end;

var
   voie: indiceVoie;
begin
   if (modeSynchro = s_seuil) and (modeAcq in [AcqTemps, AcqUnMega]) and
      not voieSynchroOK(FvoieSynchro) then begin
      if modeAcq = AcqUnMega then begin
         FvoieSynchro := voieA;
         voieOrphy[voieA].numOrphy := numSynchroGTS2;
         voieOrphy[voieA].active := True;
      end;
      for voie := voieA to voieAnaMax do
         if voieSynchroOK(voie) then
            FvoieSynchro := voie;
      if not voieSynchroOK(FvoieSynchro) then begin
         voie := voieA;
         while (voie < voieAnaMax) and voieOrphy[voie].active do
            Inc(voie);
         if not isGTI2 then
            voieOrphy[voie].numOrphy := numSynchroGTS2;
         voieOrphy[voie].active := True;
         FvoieSynchro := voie;
      end;
   end;
   Result := FvoieSynchro;
end;

function TgrapheGTS.SeuilSynchroReal: double;
begin
   if (voieSynchro >= voieA) and (voieSynchro <= voieAnaMax) then
      with voieOrphy[voieSynchro] do
         if isGTI2 then
            Result := valeurReal(0) + (valeurReal(HardMaxi) - valeurReal(-HardMaxi)) *
               (SeuilSynchro - 2048) / 4096
         else
            Result := valeurReal(0) + (valeurReal(HardMaxi) - valeurReal(-HardMaxi)) *
               (SeuilSynchro - 128) / 256
   else
      Result := 0;
end;

procedure TgrapheGTS.setSeuilSynchro(sync: double);
begin
   if (voieSynchro >= voieA) and (voieSynchro <= voieAnaMax) then
      with voieOrphy[voieSynchro] do
         if isGTI2 then begin
            SeuilSynchro := 2048 + round((sync - valeurReal(0)) * 4096 /
               (valeurReal(HardMaxi) - valeurReal(-HardMaxi)));
            if seuilSynchro > 4000 then
               seuilSynchro := 4000;
            if seuilSynchro < 96 then
               seuilSynchro := 96;
         end
         else begin
            SeuilSynchro := 128 + round((sync - valeurReal(0)) * 256 /
               (valeurReal(HardMaxi) - valeurReal(-HardMaxi)));
            if seuilSynchro > 250 then
               seuilSynchro := 250;
            if seuilSynchro < 5 then
               seuilSynchro := 5;
         end;
end;

procedure Tmagnum.setModeAngulaire(Value: boolean);
begin
   if FmodeAngulaire <> Value then begin
      FmodeAngulaire := Value;
      if Value then begin
         if modeRadian then
            nomUnite := 'rad'
         else
            nomUnite := '';
      end
      else
      begin
         TraitMagnumE := traitMagnum;
         nomUnite := 'm';
      end;
   end;
end;

procedure Tmagnum.setChute(Value: boolean);
begin
   if Fchute <> Value then begin
      Fchute := Value;
      if Value then begin
         FmodeAngulaire := False;
         nomUnite := 'm';
      end
      else if FmodeAngulaire then
      else
         TraitMagnumE := traitMagnum;
   end;
end;

procedure Tmagnum.setModeRadian(Value: boolean);
begin
   if (FmodeRadian <> Value) and
       FmodeAngulaire then begin
      FmodeRadian := Value;
      if Value
         then nomUnite := 'rad'
         else nomUnite := '';
   end;
end;

function TsortieAnalogique.CodeWGO: string;
begin
   Result := 'WGENEACQUI ' + SAi + ' ' + IntToStr(FinSortie[1]) +
      ' ' + IntToStr(NbrePulse) + ' ' + IntToStr(LargeurPulse) + ' ' +
      IntToStr(DistancePulse) + ' ' + IntToStr(DebutPulse);
   if base32Micro
      then Result := Result + ' 1'
end;

function TsortiePWM.CodePWM: string;
begin
   setPeriode;
   if not FperiodeOK then
      Result := 'WPWM'
   else
      Result := 'WPWM ' + PWMi + ' ' + IntToStr(FcodeDuty) + ' ' +
         IntToStr(Fperiode) + ' ' + IntToStr(Fhorloge);
end;

procedure TsortiePWM.setPeriode;
var
   i: integer;
begin
   Fperiode := HorlogeMaxPWM div Ffrequence;
   Fhorloge := 0;
   while (Fperiode > 65535) and (Fhorloge < 4) do begin
      Inc(Fhorloge);
      Fperiode := Fperiode div 2;
   end;
   FcodeDuty  := (Fduty100 * Fperiode) div 100;
   FperiodeOK := Fperiode < 65535;
   Ffrequence := HorlogeMaxPWM div Fperiode;
   for i := 0 to pred(Fhorloge) do
      Ffrequence := Ffrequence div 2;
   Fduty100 := FcodeDuty * 100 div Fperiode;
end;

procedure TgrapheGTS.setNbreInter(Avalue: integer);
var
   voie: indiceVoie;
   Nmax: integer;
begin
   FnbreInter := Avalue;
   if ModeAcq = AcqMagnum then
      exit;
   NbreVoie := 0;
   for voie := voieA to voieAnaMax do
      if voieOrphy[voie].active then
         Inc(NbreVoie);
   if NbreVoie = 0 then
   begin
      voieOrphy[voieA].active := True;
      NbreVoie := 1;
   end;
   Nmax := MaxResulGTS div NbreVoie;
   if Nmax > maxVecteur then
      Nmax := MaxVecteur;
   if Avalue > NMax then
      Avalue := NMax;
   if Avalue < MinPointsTemps then
      Avalue := MinPointsTemps;
   if ModeAcq = AcqUnMega then begin
      NbreTrame := 10 * NbreVoie * FreqMega;
      Avalue := NbreTrame * (Avalue div NbreTrame);
      NbreParVoieParTrame := Avalue div NbreTrame;
   end;
   FnbreInter := Avalue;
end;

procedure TbaseDeTemps.setDureeBalayage(Avalue: double);
var
   dureeMin, dureeMax: double;
begin
   if grapheGTS.ModeAcq = AcqUnMega then begin
      dureeMin := 50e-6;
      dureeMax := duree125(MaxVecteur * 1e-6 / FreqMega);
   end
   else begin
      if GrapheGTS.NbreVoie = 1 then
         dureeMin := 1e-4
      else
         dureeMin := minPointsTemps * GrapheGTS.NbreVoie * deltatOrphy;
      dureeMax := dureeMaxMax;
   end;
   if Avalue < dureeMin then
      Avalue := dureeMin;
   if Avalue > dureeMax then
      Avalue := dureeMax;
   FdureeBalayage := avalue;
   if (FPreAcq100 > 0) and PreAcqPermis and (grapheGTS.modeAcq = AcqTemps) and
      (grapheGTS.modeSynchro in [s_seuil, s_front]) then begin
      mini := -FdureeBalayage * PreAcq100 / 100;
      maxi := FdureeBalayage + mini;
   end
   else begin
      mini := 0;
      maxi := FdureeBalayage;
   end;
end;

procedure TbaseDeTemps.setPreAcq100(Avalue: integer);
var
   FnbrePreAcq: integer;
begin
   if not PreAcqPermis then
      Avalue := 0;
   FnbrePreAcq := (Avalue * grapheGTS.NbreInter) div 100;
   if FnbrePreAcq > MaxPreAcq then
      Avalue := (MaxPreAcq * 100) div grapheGTS.NbreInter;
   FPreAcq100 := Avalue;
   setDureeBalayage(FdureeBalayage);
end;

function TbaseDeTemps.getNbrePreAcq: integer;
var
   FnbrePreAcq: integer;
begin
   if (grapheGTS.modeAcq = AcqTemps) and PreAcqPermis and
      (grapheGTS.modeSynchro in [s_seuil, s_front]) and (FpreAcq100 > 0) then
   begin
      FnbrePreAcq := (FpreAcq100 * grapheGTS.NbreInter) div 100;
      if FnbrePreAcq > MaxPreAcq then
         FNbrePreAcq := MaxPreAcq;
      Result := FnbrePreAcq;
   end
   else
      Result := 0;
end;

{$I Gauss}

function Tmagnum.AffVoltmetre: string;
begin
   if isCodeurInc and CodeurActif then
      Result := inherited AffVoltmetre + ' ' + nomUnite
   else
      Result := '';
end;

function Tfrequence.AffVoltmetre: string;
var tampon : string;
    unite : coefUnite;
    exponent : integer;
begin
if valeurCourante<1
  then result := ''
  else begin
     Exponent := floor(Log10(valeurCourante));
     if valeurCourante<1000
        then begin
           unite := nulle;
        end
        else if valeurCourante<1e6
             then begin
                exponent := exponent-3;
                unite := kilo;
             end
             else begin
                exponent := exponent-6;
                unite := mega;
             end;
        tampon := FloatToStrF(valeurCourante/valUnite[unite],ffFixed,precisionGTS,3-exponent);
        result := tampon+' '+nomAff(puissanceUnite[unite]);
    end;
end;

function TvoieCalculee.CalcValeur: double;
var
   v: indiceVoie;
begin
   if parser <> nil then begin
      for v := voieT to voieAnaMax do
         if (grapheGTS.voieOrphy[v].active) and (VoltEAParser[v] <> nil) then
            VoltEAParser[v]^ := grapheGTS.voieOrphy[v].valeurCourante;
      try
         Result := Parser.Value;
      except
         Result := sansSignif;
      end;
   end
   else
      Result := sansSignif;
end;

procedure TvoieCalculee.RecopieC(N: integer);
var
   v: indiceVoie;
   i: integer;
   valeur: double;
begin
   for i := 0 to Pred(N) do begin
      for v := voieT to voieAnaMax do
         if grapheGTS.voieOrphy[v].active and (VoltEAParser[v] <> nil) then
            VoltEAParser[v]^ := grapheGTS.voieOrphy[v].vecteurAff[i];
      try
         valeur := Parser.Value;
      except
         valeur := sansSignif;
      end;
      vecteurAff[i] := valeur;
   end;
end;

procedure TvoieCalculee.setExpression(Avalue: string);

procedure setNomVoies;
var
   v: indiceVoie;
begin
   if parser <> nil then
      for v := voieT to voieAnaMax do
         if (grapheGTS.voieOrphy[v].active) then begin
            try
               VoltEAParser[v] := Parser.SetVariable(grapheGTS.voieOrphy[v].nom, 1);
            except
               VoltEAParser[v] := nil;
            end;
         end
         else VoltEAParser[v] := nil;
end;

begin
   if Avalue <> '' then begin
      if parser = nil then
         Parser := Tparser.Create(nil);
      setNomVoies;
      try
         trimComplet(Avalue);
         Parser.expression := Avalue;
      except
         if active then
            MessageDlg(Avalue + ' : erreur de syntaxe', mtError, [mbOK, mbHelp],HELP_VoieCalculee);
         active := False;
      end;
   end
   else if parser <> nil then
      Parser.expression := '';
end;

procedure TvoieCalculee.Load(Avoie: string; fichier: tMemIniFile);
begin
   with fichier do
   begin
      Nom  := ReadString(Avoie, 'Nom', Nom);
      NomUnite := ReadString(Avoie, 'Unite', NomUnite);
      Signif := ReadString(Avoie, 'Signif', '');
      Mini := ReadFloat(Avoie, 'Mini', -5);
      Maxi := ReadFloat(Avoie, 'Maxi', 5);
      active := ReadBool(Avoie, 'Active', False);
      Couleur := readInteger(Avoie, 'Couleur', couleur);
      Motif := Tmotif(readInteger(Avoie, 'Motif', 0));
      Expression := ReadString(Avoie, 'Expression', '');
      active := active and (expression <> '');
      setMinMax;
   end;
end;

procedure TvoieCalculee.Store(Avoie: string; Fichier: tMemIniFile);
begin
   with fichier do
   begin
      WriteString(Avoie, 'Nom', nom);
      WriteString(Avoie, 'Unite', nomUnite);
      WriteString(Avoie, 'Signif', signif);
      WriteFloat(Avoie, 'Mini', mini);
      WriteFloat(Avoie, 'Maxi', maxi);
      WriteBool(Avoie, 'Active', active);
      WriteInteger(Avoie, 'Couleur', couleur);
      WriteInteger(Avoie, 'Motif', Ord(motif));
      WriteString(Avoie, 'Expression', Expression);
   end;
end;

function TGrapheGTS.ordreLanceFiltre: string;
begin
   // WPE 2 30000 20 1 Ai Aj<cr>
   Result := 'WPE 2 30000 ' + IntToStr(deltatFiltre) + ' 1 ' +
      IntToStr(voieOrphy[voieFiltreEntree].numOrphy) + ' ' +
      IntToStr(voieOrphy[voieFiltreSortie].numOrphy);
end;

procedure TGrapheGTS.Print;
const
   MargeImprimante = 21;
   MargeImprimantePaysage = 30; // soit 1 cm
   HauteurGr = 0.43; // Hauteur graphe seul 
var
   margeI, hauteur: integer;
begin
   printer.beginDoc;
   limiteFenetre := Printer.Canvas.ClipRect;
   if printer.orientation = poPortrait then
      margeI := Printer.PageWidth div margeImprimante
   else
      margeI := Printer.PageWidth div margeImprimantePaysage;
   if printer.orientation = poLandscape then
      with limiteFenetre do
      begin
         top  := top + margeI div 2;
         bottom := bottom - margeI;
         right := right - margeI;
         left := left + margeI;
      end
   else
      with limiteFenetre do begin
         right := right - margeI;
         left := left + margeI;
         hauteur := round((bottom - top) * hauteurGr);
         top := top + margeI div 2;
         bottom := top + hauteur;
      end;
   canvas := Printer.Canvas;
   Monochrome := GetDeviceCaps(Printer.handle, NumColors) <= 2;
   ImpressionEnCours := True;
   Printer.Canvas.Font.Color := clBlack;
   draw;
   ImpressionEnCours := False;
   Monochrome := False;
   printer.endDoc;
end;

procedure TGrapheFourier.Print;
const
   MargeImprimante = 21;
   MargeImprimantePaysage = 30; // soit 1 cm
   HauteurGr = 0.43; // Hauteur graphe seul
var
   margeI, hauteur: integer;
begin
   printer.beginDoc;
   limiteFenetre := Printer.Canvas.ClipRect;
   if printer.orientation = poPortrait then
      margeI := Printer.PageWidth div margeImprimante
   else
      margeI := Printer.PageWidth div margeImprimantePaysage;
   if printer.orientation = poLandscape then
      with limiteFenetre do begin
         top  := top + margeI div 2;
         bottom := bottom - margeI;
         right := right - margeI;
         left := left + margeI;
      end
   else
      with limiteFenetre do begin
         right := right - margeI;
         left := left + margeI;
         hauteur := round((bottom - top) * hauteurGr);
         top := top + margeI div 2;
         bottom := top + hauteur;
      end;
   canvas := Printer.Canvas;
   Monochrome := GetDeviceCaps(Printer.handle, NumColors) <= 2;
   ImpressionEnCours := True;
   Printer.Canvas.Font.Color := clBlack;
   draw;
   ImpressionEnCours := False;
   Monochrome := False;
   printer.endDoc;
end;

procedure TGrapheGTS.VersBitMap(NomFichier: string; isJpeg: boolean);
const
   XmaxBitmap: integer = 640;
   YmaxBitmap: integer = 400; {16/10}
var
   AbitMap: TbitMap;
   Ajpeg: TjpegImage;
begin
   AbitMap := TbitMap.Create;
   AbitMap.Height := YmaxBitmap;
   AbitMap.Width := XmaxBitmap;
   AbitMap.pixelFormat := pf24bit;
   setRect(limiteFenetre, 0, 0, XmaxBitmap, YmaxBitmap);
   canvas := AbitMap.Canvas;
   Draw;
   if isJpeg then begin
      Ajpeg := TjpegImage.Create;
      Ajpeg.Assign(Abitmap);
      Ajpeg.compress;
      if NomFichier = '' then
         ClipBoard.Assign(Ajpeg)
      else
         Ajpeg.saveToFile(NomFichier);
      Ajpeg.Free;
   end
   else if NomFichier = '' then
      ClipBoard.Assign(AbitMap)
   else
      AbitMap.saveToFile(NomFichier);
   AbitMap.Free;
end;

procedure TGrapheFourier.VersBitMap(NomFichier: string; isJpeg: boolean);
const
   XmaxBitmap: integer = 640;
   YmaxBitmap: integer = 400; {16/10}
var
   AbitMap: TbitMap;
   Ajpeg: TjpegImage;
begin
   AbitMap := TbitMap.Create;
   AbitMap.Height := YmaxBitmap;
   AbitMap.Width := XmaxBitmap;
   AbitMap.pixelFormat := pf24bit;
   setRect(limiteFenetre, 0, 0, XmaxBitmap, YmaxBitmap);
   canvas := AbitMap.Canvas;
   Draw;
   if isJpeg then begin
      Ajpeg := TjpegImage.Create;
      Ajpeg.Assign(Abitmap);
      Ajpeg.compress;
      if NomFichier = '' then
         ClipBoard.Assign(Ajpeg)
      else
         Ajpeg.saveToFile(NomFichier);
      Ajpeg.Free;
   end
   else if NomFichier = '' then
      ClipBoard.Assign(AbitMap)
   else
      AbitMap.saveToFile(NomFichier);
   AbitMap.Free;
end;

procedure Tcourbe.chercheMinmax(Nbre: integer; var min, max: double);
var
   i: integer;
   marge: double;
begin
   Min := vecteurAff[0];
   Max := Min;
   for i := 1 to pred(Nbre) do
      if vecteurAff[i] < Min then
         Min := vecteurAff[i]
      else if vecteurAff[i] > Max then
         Max := vecteurAff[i];
   marge := (maxi - mini) / 50;
   min := min - marge;
   if min < mini then
      min := mini;
   max := max + marge;
   if max > maxi then
      max := maxi;
end;

procedure TgrapheGTS.PenDefault;
begin
   canvas.pen.mode  := pmCopy;
   canvas.pen.style := psSolid;
   if impressionEnCours then
      canvas.pen.Width := 3
   else
      canvas.pen.Width := 1;
end;

procedure TgrapheFourier.PenDefault;
begin
   canvas.pen.mode  := pmCopy;
   canvas.pen.style := psSolid;
   if impressionEnCours then
      canvas.pen.Width := 3
   else
      canvas.pen.Width := 1;
end;

procedure Tmagnum.setOrigine(var N : integer);
var
   i,k,imax : integer;
   debut : double;
begin
      i := 0;
      imax := N div 3;
      if montant then begin
         repeat inc(i);
         until (i>imax) or
               ( (vecteurAff[i-1]<vecteurAff[i]) and
                 (vecteurAff[i-1]*vecteurAff[i]<=0) );
      end
      else begin
          repeat inc(i);
          until (i>imax) or
               ( (vecteurAff[i-1]>vecteurAff[i]) and
                 (vecteurAff[i-1]*vecteurAff[i]<=0) );
      end;
      if vecteurAff[i-1]=0 then i := i-1;
      N := N-i;
      debut := BaseDeTemps.vecteurAff[i];
      for k := 0 to N do begin
          vecteurAff[k] := vecteurAff[k+i];
          BaseDeTemps.vecteurAff[k] := BaseDeTemps.vecteurAff[k+i]-debut;
      end;
end;

Function TgrapheFourier.isAxeX(x,y : integer) : boolean;
begin
    result := (y>limiteCourbe.bottom) and
              (y<(limiteCourbe.bottom+hautCarac)) and
              (x>limiteCourbe.left) and
              (x<(limiteCourbe.right-8*largCarac))
end;

procedure TgrapheFourier.changeEchelleX(xnew,xold : integer);
begin with voieT do begin
     xnew := xnew-limiteCourbe.left;
     if xnew<16 then exit;
     xold := xold-limiteCourbe.left;
     A := A*xnew/xold;
     Maxi := (limiteCourbe.right-limiteCourbe.left)/A;
end end;

Procedure TgrapheFourier.PointProche(var x,y : Integer;var indexVoie,indexPoint : integer);
var
   i : integer;
   Xi,Yi,newX,newY : Integer;
   distance,distanceMin : integer;
   voie : indiceVoie;
begin
  indexPoint := -1;
  newX := x;
  newY := y;
  distanceMin := 32;
  if y>(limiteCourbe.bottom-4) then exit;
  for i := 0 to pred(NbreAff) do begin
        with voieT do xi := Xwindow(vecteurAff[i]);
        for voie := voieA to VoieAnaMax do
            with voieOrphy[voie] do if active then begin
            yi := Xwindow(vecteurAff[i]);
            distance := abs(X-Xi)+abs(Y-Yi);
            if (distance<distanceMin) then begin
               distanceMin := distance;
               indexPoint := i;
               newX := Xi;
               newY := Yi;
               indexVoie := voie;
            end;
            end;
      end;
      if indexPoint>=0 then begin
         x := newX;
         y := newY;
      end;
end;

Procedure TgrapheGTS.SetIndexPeriode(voie : indiceVoie;var indexPeriode : integer);
var i,debut,fin : integer;
    ref,ecart,ecartMax : double;
begin
         ref := voieOrphy[voie].vecteurAff[0];
         debut := indexPeriode-(indexPeriode div 20);
         fin := indexPeriode+(indexPeriode div 20);
         if debut<0 then debut := 0;
         if fin>=nbreAff then fin := pred(NbreAff);
         ecartMax := voieOrphy[voie].maxi/10;
         for i := debut to fin do begin
             ecart := abs(voieOrphy[voie].vecteurAff[i]-ref);
             if ecart<ecartMax then begin
                ecartMax := ecart;
                indexPeriode := i;
             end;
         end;
end;

procedure TgrapheGts.TraceRepere(repere: Trepere);
var
   x,largeur: integer;
begin with Canvas, repere do begin
      Pen.color := ColorGr(couleurRepere);
      Pen.Style := psDash;
      Pen.Width := 1;
      x := VoieOrphy[voieT].Xwindow(valeur);
      MoveTo(x, LimiteCourbe.top);
      LineTo(x, LimiteCourbe.bottom);
      largeur := textWidth(edit.text);
      edit.Left := x-(largeur div 2);
      edit.Top := (3*limiteCourbe.Top+limiteCourbe.Bottom) div 4;
      edit.autosize := largeur>80;
      if largeur<80 then edit.width := 80;
      PenDefault;
end end;{ TraceRepere }

procedure TlisteRepere.Clear;
var
  i: integer;
begin
  for i := 0 to pred(Count) do
    items[i].Free;
  inherited Clear;
end;

function TlisteRepere.Remove(Item: TRepere): integer;
begin
  Result := inherited Remove(Item);
  Item.Free;
end;

function TlisteRepere.Add(Item: TRepere): integer;
begin
  Result := inherited Add(Item);
end;

function TlisteRepere.Get(Index: integer): TRepere;
begin
  Result := TRepere(inherited Get(Index));
end;

procedure TlisteRepere.Put(Index: integer; Item: TRepere);
begin
  inherited Put(Index, Item);
end;

destructor Trepere.Destroy;
begin
    edit.free;
    inherited destroy;
end;

end.

