unit GTI2comp;

interface
uses Windows, sysUtils, Messages, gti2_dynamique, oomisc, gtsutil, graphe;

const
    WM_Zresul = WM_USER + 16;
                                                  
type
    indiceHorloge = 0..2;                                             
    TRAMGTI2 = array[0..60000] of AnsiChar;
    VecteurGBF = array[0..NbreMaxGBF] of LongInt; // 32 bits

    TGTI2 = class(Tobject)
          private                                           
              FhandleGTI2 : Thandle;
              FThreadResul : Thandle;
              FhandleWindow : HWND;
              Fversion : shortString;
              DataGet : Tdata_RS232;
              FValeurMot : pseudoWord;
              Fnbre : integer;
              Findex : integer;
              FnbreResul,FunPoint : integer;
              Fstop : boolean;
              Fdata : TRAMGTI2;

              FreloadGBF : integer;
              FdataGenerateur : Data_Generateur;
              FfreqGBF : integer;
              FhorlogeGBF : indiceHorloge;
              FNbreGBF : integer;

              function getResul : integer;
              Procedure send(commande : shortString);
              procedure setFreqGBF(afreq : integer);
          public
              Property output : shortString write send;
              // gti2.ouput := 'ZEA' envoie 'ZEA' en ajoutant le retour chariot
              // pour compatibilit de vocabulaire avec gestion voie srie
              property NbreGBF : integer read FNbreGBF;
              property FreqGBF : integer read FFreqGBF write setFreqGBF;
              procedure LanceGenerateur(const data : vecteurGBF);
              procedure StopGenerateur;
              function Open : boolean;
              // pour ouvrir, rpond oui si celle-ci s'est passe correctement
              procedure Close;
              function Version : shortString;
              function getChar : AnsiChar;
              // pour compatibilit de vocabulaire avec gestion voie srie
              function getValeurMot(commande : shortString;attente,count : integer) : pseudoWord;
              // Utilisation
              // pour un word : GetValeurMot(commande,5,2); result := valeurMot.mot;
              // pour un longWord : GetValeurMot(commande,5,4); result := valeurMot.LongMot;
              // pour un octet : GetValeurMot(commande,5,1); result := valeurMot.octet;
              // pour un entier : GetValeurMot(commande,5,2); result := valeurMot.entier;
              // L'attente est un time out, il est en paramtre et non fig, pour le cas particulier
              // mesure de frquence dans laquelle la rponse peut tre non instantane
              Function getReco : pseudoWord;
              procedure LanceThreadZresul(nbreBytes,nbreVoies : integer);
              // Lance aprs une demande de 'Zresul' un thread d'attente des rsultats
              //  chaque rception de donnes, envoie  la fentre un message Zresul
              //  la fin de la rception envoie un message finZresul pour que le programme
              // ferme le thread proprement
              procedure FlushInBuffer;
              procedure FlushOutBuffer;              
              procedure StopThreadZresul(arret : boolean);
              // arrt du thread Zresul, arret pour envoi de WPE
              procedure StopThreadMagnum;
              // arrt du thread dans le cas particulier Magnum
              constructor Create(ahandle : HWND);
              // handle de la fentre principale
              destructor Destroy;override;
          end;
                                                     

const FreqHorloge : array[indiceHorloge] of integer = (48000000,12000000,4000000);

var GTI2 : TGTI2;
{$IFDEF debug}
    FichierMagnum : TextFile;
    FichierMagnumActif : boolean; 
{$ENDIF}

implementation

function GetResulThread : integer;stdcall;
begin
    result := gti2.getResul
end;

function TGTI2.Open : boolean;
begin
     if FhandleGTI2<=0 then begin
        FHandleGTI2 := _SetPortOpen('OrphyGTI2-0');
        if FhandleGTI2<=0 then
           FHandleGTI2 := _SetPortOpen('OrphyGTI2-1');
        if FhandleGTI2>0 then begin
            sleep(50);
//            _ChangeVitesse(FHandleGTI2,1); // 125 000 bauds
//            sleep(500);
            _LanceThread(FHandleGTI2);
            sleep(100);
        end;
     end;
     result := (FhandleGTI2>0) and (_isRunThread=1);
end;

procedure TGTI2.Close;
begin
     stopThreadZresul(false);
     if (FhandleGTI2>0) and (_isRunThread=1) then begin
          _StopThread;
          sleep(50);
     end;
     if FhandleGTI2>0 then begin
        _SetPortClose(FHandleGTI2);
        sleep(50);
     end;
     FhandleGTI2 := 0;
end;

function TGTI2.Version : shortString;

function DonneVersion(nomExe : string) : shortString;
var
  VerInfoSize, VerValueSize,
  Dummy      : DWORD;
  VerValue   : PVSFixedFileInfo;
  VerInfo    : Pointer;
begin
  Result := '0.0.0';
  VerInfoSize := GetFileVersionInfoSize(PChar(NomExe),Dummy);
  If VerInfoSize = 0 Then Exit;
  GetMem(VerInfo, VerInfoSize);
  try
    if VerInfo <> nil then begin
      GetFileVersionInfo(PChar(NomExe), 0, VerInfoSize,VerInfo);
      VerQueryValue(VerInfo, '\', Pointer(VerValue), VerValueSize);
      with VerValue^ do begin
        Result := Format('%d.%d.%d', [dwFileVersionMS shr 16,
        dwFileVersionMS and $FFFF,dwFileVersionLS and $FFFF ]);
      end; { with }
     end; { if }
  finally
    FreeMem(VerInfo, VerInfoSize);
  end;
end;

procedure ChercheVersion;
var version : array[0..255] of char;
begin
   _GetVersion(FhandleGTI2,version);
   Fversion := string(version);
   Fversion := Fversion+' DLL '+donneVersion('GTICOMMX.DLL');
end;

begin
   if FhandleGTI2<=0
      then result := 'GTI II non branche ?'
      else begin
         if Fversion='' then chercheVersion;
         result := Fversion;
      end;                            
end;

function TGTI2.getValeurMot(commande : shortString;attente,count : integer) : pseudoWord;
var ET : eventTimer;
    indexOctet,i,longueur : integer;
begin
    flushInBuffer;
    for i := 0 to pred(count) do
        FvaleurMot.octets[i] := #0;
    indexOctet := 0;
    send(commande);
    newTimer(ET,attente);
{$IFDEF debug}
    if FichierMagnumActif then
        write(FichierMagnum,TimeToStr(now),' getMot : ');
{$ENDIF}
    repeat
      longueur := _GetLenH8;
      if longueur>0 then begin
         if longueur>MaxDataGet then longueur := MaxDataGet;
         longueur := _GetH8(dataGet,longueur,0);
         for i := 0 to pred(longueur) do begin
             FvaleurMot.octets[indexOctet] := dataGet.bufferC[i];
             inc(indexOctet);
         end;
      end;
   until (indexOctet=count) or timerExpired(ET);
{$IFDEF debug}
   if FichierMagnumActif then begin
      for i := 0 to pred(indexOctet) do
          write(FichierMagnum,ord(FvaleurMot.octets[i]),',');
      writeln(FichierMagnum);
   end;
{$ENDIF}
   result := FvaleurMot;
end;

Function TGTI2.getReco : pseudoWord;
begin
     GetValeurMot('WRECO',8,12);
     result := FvaleurMot;               
end;

Procedure TGTI2.send(commande : shortString);
var i,longueur : integer;
//    ET : eventTimer;
begin
    longueur := length(commande);
    for i := 0 to pred(longueur) do
        dataGet.bufferC[i] := commande[i+1];
    dataGet.buffer[longueur] := 13; // fin de chaine
    if longueur>1 then longueur := longueur+1; // sinon mode rapide sur un caractre
    _SendH8(dataGet,longueur);
    sleep(longueur*2);
{$IFDEF DEBUG}
    if FichierMagnumActif then
       writeln(FichierMagnum,TimeToStr(now),' output : ',commande);
{$ENDIF}
end;

constructor TGTI2.Create(ahandle : HWND);
begin
  inherited create;
  FhandleGTI2 := 0;
  FHandleWindow := ahandle;
  Fversion := '';
end;

destructor TGTI2.Destroy;
begin
    close;
    inherited destroy;
end;

function TGTI2.getResul : integer;
var IncNbre,i : integer;
    longueur : integer;
begin
        FNbre := 0;
        Findex := 0;
        Fstop := false;
        try
        repeat
             longueur := _GetLenH8;
{$IFDEF debug}
//             if FichierMagnumActif then
//                writeln(FichierMagnum,'getlenh8=',longueur);
{$ENDIF}

             if longueur>0 then begin
                if longueur>MaxDataGet then longueur := MaxDataGet;
                longueur := FunPoint*(longueur div FunPoint);
                 // par bloc de un point soit 2 octets * le nombre de voies
                IncNbre := _GetH8(dataGet,longueur,0);
                if IncNbre>0 then begin
{$IFDEF debug}
{
                    if FichierMagnumActif then begin
                       for i := 0 to pred(incNbre) do begin
                           if (i mod 16)=15
                              then writeln(FichierMagnum,ord(dataGet.bufferC[i]))
                              else write(FichierMagnum,ord(dataGet.bufferC[i]),',');
                       end;
                       if ((incNbre-1) mod 16)<15
                              then writeln(FichierMagnum);
                    end;
}
{$ENDIF}                                                      
                    for i := 0 to pred(incNbre) do
                        Fdata[Fnbre+i] := dataGet.BufferC[i];
                    FNbre := FNbre + IncNbre;
{$IFDEF debug}
                    if FichierMagnumActif then
                       writeln(FichierMagnum,TimeToStr(now),' ',
                                 incNbre div FunPoint,' points => total : ',
                                 (Fnbre div FunPoint));
{$ENDIF}
                    if not Fstop then
                       PostMessage(FHandleWindow,WM_ZResul,IncNbre,0);
                end;
             end;
             sleep(10);
        until (FNbre >= FnbreResul) or Fstop;
{$IFDEF debug}
        if FichierMagnumActif then
              writeln(FichierMagnum,TimeToStr(now),' fin getResul');
{$ENDIF}
        except
        end;
        result := 0
end;

procedure TGTI2.LanceThreadZresul(nbreBytes,nbreVoies : integer);
var code : cardinal;
begin
      if FhandleGTI2<=0 then exit;
      try
      Fnbre := 0;
      FnbreResul := nbreBytes;
      FunPoint := 2*nbreVoies;
{$IFDEF debug}
      if FichierMagnumActif then
         writeln(FichierMagnum,TimeToStr(now),' lance thread Zresul');
{$ENDIF}
      FThreadResul := CreateThread(nil,0,@GetResulThread,nil,0,code);
      except
      end;
end;

Function TGTI2.GetChar : AnsiChar;
begin
     if Findex<Fnbre
        then begin
           result := Fdata[Findex];
           inc(Findex);
        end
        else result := #0
end;

Procedure TGTI2.StopThreadZresul(arret : boolean);
var code : cardinal;
    acqEnCours : boolean;
    ET : eventTimer;
begin
      if FhandleGTI2<=0 then exit;
      if FThreadResul<>0 then begin
{$IFDEF debug}
         if FichierMagnumActif then
            writeln(FichierMagnum,TimeToStr(now),' stop thread Zresul');
{$ENDIF}
         Fstop := true;
         if (FNbre<FnbreResul) or arret then begin
              send('WPE');
//            send('WPE 1 8 10 1 0');
//            _stopAcquiLente;  // arrt
//            _stopAcquiLente; // rarme 
         end;
         newTimer(ET,18);
         repeat
              AcqEnCours := getExitCodeThread(FThreadResul,code) and
                           (code=Still_active);
              sleep(10);
         until not AcqEnCours or TimerExpired(ET);
         if FNbre<FnbreResul then flushInBuffer;
         closeHandle(FThreadResul);
         FThreadResul := 0;
     end;
end;

Procedure TGTI2.StopThreadMagnum;
var code : cardinal;
    acqEnCours : boolean;
    ET : eventTimer;
begin
      if FhandleGTI2<=0 then exit;
      if FThreadResul<>0 then begin
         Fstop := true;
         newTimer(ET,18);
         repeat
              AcqEnCours := getExitCodeThread(FThreadResul,code) and
                           (code=Still_active);
         until not AcqEnCours or TimerExpired(ET);
         if FNbre<FnbreResul then flushInBuffer;
         closeHandle(FThreadResul);
         FThreadResul := 0;
     end;
end;

procedure TGTI2.LanceGenerateur(const data : vecteurGBF);
var i : integer;
begin
    _StopThread;
    sleep(20);
    _StopGenerateur(FhandleGTI2);
    FdataGenerateur.NbreData := FnbreGBF;
    FdataGenerateur.BaseDeTemps := FhorlogeGBF;
    FdataGenerateur.Reload := FreloadGBF;
    for i := 0 to pred(NbreMaxGBF) do
        FdataGenerateur.bufferOut[i] := data[i];
    _LanceGenerateur(FhandleGTI2,1,FdataGenerateur);
    _LanceThread(FHandleGTI2);
    sleep(20);
end;

procedure TGTI2.StopGenerateur;
begin
    _StopGenerateur(FhandleGTI2)
end;

procedure TGTI2.setFreqGBF(afreq : integer);
begin
    if afreq<1 then afreq := 1;
    if afreq>=100000 then afreq := 99999;
    FReloadGBF := 0;
    FhorlogeGBF := 0;         // 48 MHz
    FNbreGBF := freqHorloge[0] div afreq;
    if FNbreGBF<NbreMaxGBF then begin
       FfreqGBF := freqHorloge[0] div FNbreGBF;
       exit;    // 3 kHz  48 MHz
    end;
    FreloadGBF := 65536-1;
    FhorlogeGBF := 1; // 12 MHz
    FNbreGBF := freqHorloge[1] div afreq;
    if FNbreGBF<NbreMaxGBF then begin
       FfreqGBF := freqHorloge[1] div FNbreGBF;
       exit; // 750 Hz  3 kHz
    end;
    FhorlogeGBF := 2;
    FNbreGBF := freqHorloge[2] div afreq;// 4 MHz
    if FNbreGBF<NbreMaxGBF then begin
         FfreqGBF := freqHorloge[2] div FNbreGBF;
         exit; // 250 Hz  750 Hz
    end;
    FReloadGBF := (FNbreGBF div NbreMaxGBF) + 1;
    FNbreGBF := FNbreGBF div FreloadGBF;
    FfreqGBF := freqHorloge[FhorlogeGBF] div FNbreGBF div FReloadGBF;
    FReloadGBF := 65536-FreloadGBF;
end;

Procedure TGTI2.flushInBuffer;
var ET : eventTimer;
    longueur : integer;
begin
    newTimer(ET,18); // 1 seconde
{$IFDEF DEBUG}
    if FichierMagnumActif then
       writeln(FichierMagnum,TimeToStr(now),' Flush in buffer');
{$ENDIF}
    repeat
        sleep(20); // ms
        longueur := _GetLenH8;
        if longueur>0 then begin
           if longueur>MaxDataGet then longueur := MaxDataGet;
           _GetH8(dataGet,longueur,0);
        end;
    until (longueur=0) or timerExpired(ET);
end;

Procedure TGTI2.FlushOutBuffer;
var ET : eventTimer;
begin
    newTimer(ET,5);
    repeat sleep(5)
    until (_emptyBuffer=0) or timerExpired(ET);
end;

end.


