Unit Maths;

Interface

uses Windows,SysUtils,math;

const
     MaxVecteur = 60000;
     RadianVersDegre = 57.29577951308232090;
     DegreVersRadian = 0.017453292519943296;
     Loge = 0.434294481903251828;
     Ln10 = 2.302585092994045680;

type
     IndiceVecteur = 0..MaxVecteur;
     Vecteur = array[IndiceVecteur] of double;
     VecteurInt = array[IndiceVecteur] of smallInt;
     VecteurWord = array[IndiceVecteur] of word;
     VecteurByte = array[IndiceVecteur] of byte;

const
     MaxReal : double = 1e+35;
     MinReal : double = 1e-35;
     ProcheReal : double = 1e-6;
     SansSignif : double = 1e-26;

Function Dix(x : integer) : double;
Function PuissDeux(X:integer) : integer; { X^N avec N>=0 }
Function Puiss2Sup(n : integer) : integer;
Function Puiss2Inf(n : integer) : integer;
Function Atan2(im,re : double) : double;
Function Log(x : double) : double;
function sinc(x : double) : double;
Procedure GetMinMax(const V : vecteur;N : indiceVecteur;var mini,maxi : double);
Procedure VerifMinMaxInt(var mini,maxi : integer);
Procedure VerifMinMaxDouble(var mini,maxi : double);

Implementation

Function sinc(x : double) : double;
begin
   try
      x := pi*x;
      sinc := sin(x)/x
   except
      sinc := 1
   end;
end;

function Dix(x : integer) : double;
Begin
    Dix := Power(10,X)
end;

Procedure VerifMinMaxInt(var mini,maxi : integer);
var z : integer;
begin
   if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
   end;
end;

Procedure VerifMinMaxDouble(var mini,maxi : double);
var z : double;
begin
   if Maxi<Mini then begin
	  z := Maxi;
	  Maxi := Mini;
	  Mini := z;
   end;
end;

Procedure GetMinMax(const V : vecteur;N : indiceVecteur;var mini,maxi : double);
var i : indiceVecteur;
begin
     Mini := V[0];
     Maxi := V[0];
     for i := 1 to pred(N) do
         if V[i]>Maxi
            then Maxi := V[i]
            else if V[i]<Mini
               then Mini := V[i]
end;

Function Atan2(im,re:double) : double;
{ z = re + j*im }
var valeur : double;
begin
   try
   valeur := arcTan(im/re);
   if re<0 then if im<0
       then valeur := valeur-pi
       else valeur := valeur+pi;
   atan2 := valeur*RadianVersDegre;
   except
         on EZeroDivide do begin
            if im<0
	       then valeur := -pi/2
               else valeur := pi/2;
            atan2 := valeur*RadianVersDegre
        end;
   end;
end;

function Puiss2Inf(N : integer) : integer;
begin
    result := 1;
    while result<N do result := 2*result;
    if result>N then result := result div 2;
end;

function Puiss2Sup(N : integer) : integer;
begin
    result := Puiss2Inf(N);
    if result<N then result := 2*result
end;

Function Log(X:double):double;
Begin
   Log := Ln(X)*loge
End;

Function PuissDeux(X:integer) : integer; { 2^N avec N>=0 }
var Y,i : integer;
begin
    Y := 1;
    for i := 1 to X do Y := Y*2;
    result := Y;
end;

end.

