unit OxymetreU;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Math, ExtCtrls, ComCtrls, StdCtrls,
  gtsUtil, gtsdef;

type
  TOxymetreEtalon = class(TForm)
    Gauge: TProgressBar;
    Timer: TTimer;
    Button1: TButton;
    Memo1: TMemo;
    Button2: TButton;
    StatusBar1: TStatusBar;
    procedure FormActivate(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  private
     valeurC : integer;
     codeVoie : string;
     abandon : boolean;
     procedure Attente;
     procedure ReglePotar;
  public
     voieCourante : Tvoie;
  end;

var
  OxymetreEtalon: TOxymetreEtalon;

implementation

uses gtsmain;

{$R *.dfm}

const Pcourant = 3;

procedure TOxymetreEtalon.attente;
var zT,oldzT : integer;
    i : integer;
begin with oxymetre do begin
    GtsMainForm.EcritOrphy('WRPOTAR '+code);
    GtsMainForm.EcritOrphy(mesureC); // mesure O2
    zT := 0;
    gauge.max := 15000 div timer.Interval; // 15 s
    gauge.position := 0;
    statusBar1.simpleText := 'Attente de stabilisation en concentration (<15 s)';
    sleep(1000);
    timer.enabled := true;
    repeat
         oldzT := zT;
         zT := 0;
         for i := 0 to 9 do begin
             zT := zT + GtsMainForm.getInt(codeVoie);
             sleep(30);
         end;
         sleep(300);
         application.ProcessMessages;
    until (gauge.position=gauge.max) or
          (abs(zT-oldzT)<32) or abandon; // 1 pour mille
    timer.enabled := false;
    if abandon then exit;

    GtsMainForm.EcritOrphy(mesureT); // mesure temprature
    gauge.Max := 5000 div timer.Interval; // 5 s
    gauge.Position := 0;
    statusBar1.simpleText := 'Attente de stabilisation en temprature (5 s)';
    timer.enabled := true;
    repeat
         application.ProcessMessages;
    until (gauge.position=gauge.max) or abandon;
    timer.enabled := false;
    if abandon then exit;
    zT := GtsMainForm.getInt(codeVoie);
    Temp := temperature(zT);
    if (Temp=0) or (Temp=100) then begin
       afficheErreurStr('Impossible de mesurer la temprature');
       Temp := 20;
    end;
    valeurC := valeurAttendue;
    GtsMainForm.EcritOrphy(mesureC); // mesure O2
    gauge.Max := 10000 div timer.Interval;
    gauge.Position := 0;
    statusBar1.simpleText := 'Attente de repolarisation (10 s)';
    timer.enabled := true;
    repeat
         application.ProcessMessages;
    until (gauge.position=gauge.max) or abandon;
    timer.enabled := false;
end end;

procedure TOxymetreEtalon.reglePotar;
var valeurP,valeurLue : integer;
    maxi,mini : integer;

function IsGreater : boolean;
const
    attenteMin = 5; { 0,5 seconde }
    attenteMax = 20; { 2 secondes }
var oldzi,i : integer;
begin with listeCapteursAuto[indexCapteurOxygene] do begin
     GtsMainForm.EcritOrphy('WEPOTAR '+oxymetre.code+' 3 '+intToStr(valeurP));
     sleep(50);
     valeurLue := 40000;
     i := 0;
     repeat
        oldzi := valeurLue;
        sleep(50); // 50 ms
        valeurLue := GtsMainForm.getInt(codeVoie);
        inc(i);
     until ((abs(valeurLue-oldzi)<8) and (i>attenteMin)) or (i>attenteMax);
     result := valeurLue>valeurC;
end end;

begin
    maxi := 1024;
    mini := 0;
    gauge.max := 10;  // 1024 = 10 bits
    gauge.position := 0;
    statusBar1.simpleText := 'Etalonnage du gain en cours';
    repeat
         valeurP := (mini+maxi) div 2;
         if isGreater
            then maxi := valeurP
            else mini := valeurP;
         gauge.position := gauge.position + 1;
         gauge.refresh;
    until (maxi-mini)<2;
    if (valeurP<16) or (valeurP>1008) then begin
       GtsMainForm.EcritOrphy('WEPOTAR '+oxymetre.code+' 3 '+intToStr(512));
       afficheErreurStr('Impossible d''talonner le gain (saturation)');
       abandon := true;
    end;
    GtsMainForm.EcritOrphy('WMPOTAR '+oxymetre.code+' 3');
    with ListeCapteursAuto[indexCapteurOxygene] do begin
       EtalonnageFait := not abandon;
       if not abandon then begin
          unVoltC[1] := unVoltC[0]*21/oxymetre.cRef*valeurC/valeurLue;
          voieCourante.unVolt := unVoltC[1];
          voieCourante.setMinMax;
       end;
    end;
end;

procedure TOxymetreEtalon.TimerTimer(Sender: TObject);
begin
     gauge.position := gauge.position + 1;
end;

procedure TOxymetreEtalon.Button1Click(Sender: TObject);
begin
    Attente;
    if abandon
    then modalResult := mrCancel
    else begin
       ReglePotar;
       if abandon
          then modalResult := mrCancel
          else modalResult := mrOK;
    end;
    close;
end;

procedure TOxymetreEtalon.Button2Click(Sender: TObject);
begin
	abandon := true;
    if not timer.enabled then begin
       modalResult := mrCancel;
       close;
    end;
end;

procedure TOxymetreEtalon.FormActivate(Sender: TObject);
begin
     listeCapteursAuto[indexCapteurOxygene].codeUniteC := 0;
     oxymetre.code := numToCodePotar(voieCourante.numOrphy);
     codeVoie := 'WEA '+IntToStr(voieCourante.numOrphy);
     ListeCapteursAuto[indexCapteurOxygene].EtalonnageFait := false;
     abandon := false;
end;

end.
